@@ -48,10 +48,17 @@ func (ba *Bind9App) DetectAllowedLogs() ([]string, error) {
 	return nil, nil
 }
 
+// Returns a list of the configured daemons in a given application.
+func (ba *Bind9App) GetConfiguredDaemons() []string {
+	// Bind9 is a single daemon application.
+	return []string{"named"}
+}
+
 // List of BIND 9 executables used during app detection.
 const (
 	namedCheckconfExec = "named-checkconf"
 	rndcExec           = "rndc"
+	namedExec          = "named"
 )
 
 // rndc key file name.
@@ -86,8 +93,8 @@ func NewRndcClient(ce CommandExecutor) *RndcClient {
 // Determine rndc details in the system.
 // It find rndc executable and prepare base command with all necessary
 // parameters including rndc secret key.
-func (rc *RndcClient) DetermineDetails(baseNamedDir, bind9ConfDir string, ctrlAddress string, ctrlPort int64, ctrlKey string) error {
-	rndcPath, err := determineBinPath(baseNamedDir, rndcExec)
+func (rc *RndcClient) DetermineDetails(baseNamedDir, bind9ConfDir string, ctrlAddress string, ctrlPort int64, ctrlKey string, executor storkutil.CommandExecutor) error {
+	rndcPath, err := determineBinPath(baseNamedDir, rndcExec, executor)
 	if err != nil {
 		return err
 	}
@@ -115,7 +122,7 @@ func (rc *RndcClient) SendCommand(command []string) (output []byte, err error) {
 	var rndcCommand []string
 	rndcCommand = append(rndcCommand, rc.BaseCommand...)
 	rndcCommand = append(rndcCommand, command...)
-	log.Debugf("rndc: %+v", rndcCommand)
+	log.Debugf("Rndc: %+v", rndcCommand)
 
 	return rc.execute(rndcCommand)
 }
@@ -129,7 +136,7 @@ func (rc *RndcClient) SendCommand(command []string) (output []byte, err error) {
 //		secret "OmItW1lOyLVUEuvv+Fme+Q==";
 //	};
 func getRndcKey(contents, name string) (controlKey string) {
-	pattern := regexp.MustCompile(`(?s)keys\s+\"(\S+)\"\s+\{(.*)\}\s*;`)
+	pattern := regexp.MustCompile(`(?s)key\s+\"(\S+)\"\s+\{(.*?)\}\s*;`)
 	keys := pattern.FindAllStringSubmatch(contents, -1)
 	if len(keys) == 0 {
 		return ""
@@ -142,14 +149,14 @@ func getRndcKey(contents, name string) (controlKey string) {
 		pattern = regexp.MustCompile(`(?s)algorithm\s+\"(\S+)\";`)
 		algorithm := pattern.FindStringSubmatch(key[2])
 		if len(algorithm) < 2 {
-			log.Warnf("no key algorithm found for name %s", name)
+			log.Warnf("No key algorithm found for name %s", name)
 			return ""
 		}
 
 		pattern = regexp.MustCompile(`(?s)secret\s+\"(\S+)\";`)
 		secret := pattern.FindStringSubmatch(key[2])
 		if len(secret) < 2 {
-			log.Warnf("no key secret found for name %s", name)
+			log.Warnf("No key secret found for name %s", name)
 			return ""
 		}
 
@@ -181,7 +188,7 @@ func parseInetSpec(config, excerpt string) (address string, port int64, key stri
 	pattern := regexp.MustCompile(`(?s)inet\s+(\S+\s*\S*\s*\d*)\s+allow\s*\{(?:\s*\S+\s*;\s*)+\}(.*);`)
 	match := pattern.FindStringSubmatch(excerpt)
 	if len(match) == 0 {
-		log.Warnf("cannot parse BIND 9 inet configuration: no match (%+v)", config)
+		log.Warnf("Cannot parse BIND 9 inet configuration: no match (%+v)", config)
 		return "", 0, ""
 	}
 
@@ -192,19 +199,19 @@ func parseInetSpec(config, excerpt string) (address string, port int64, key stri
 	case 3:
 		address = inetSpec[0]
 		if inetSpec[1] != "port" {
-			log.Warnf("cannot parse BIND 9 control port: bad port statement (%+v)", inetSpec)
+			log.Warnf("Cannot parse BIND 9 control port: bad port statement (%+v)", inetSpec)
 			return "", 0, ""
 		}
 
 		iPort, err := strconv.Atoi(inetSpec[2])
 		if err != nil {
-			log.Warnf("cannot parse BIND 9 control port: %+v (%+v)", inetSpec, err)
+			log.Warnf("Cannot parse BIND 9 control port: %+v (%+v)", inetSpec, err)
 			return "", 0, ""
 		}
 		port = int64(iPort)
 	case 2:
 	default:
-		log.Warnf("cannot parse BIND 9 inet_spec configuration: no match (%+v)", inetSpec)
+		log.Warnf("Cannot parse BIND 9 inet_spec configuration: no match (%+v)", inetSpec)
 		return "", 0, ""
 	}
 
@@ -212,13 +219,17 @@ func parseInetSpec(config, excerpt string) (address string, port int64, key stri
 		// Find a key clause. This pattern is build up like this:
 		// keys\s*                - keys
 		// \{\s*                  - {
-		// \"(\S+)\"\s*;          - key_list (first)
-		// (?:\s*\"\S+\"\s*;\s*)* - key_list (remainder)
-		// \s}\s*                 - }
-		pattern = regexp.MustCompile(`(?s)keys\s*\{\s*\"(\S+)\"\s*;(?:\s*\"\S+\"\s*;\s*)*\}\s*`)
+		// \"(.*?)\"\s*;\s*       - key_list (first)
+		// (?:\s*\".*?\"\s*;\s*)* - key_list (remainder)
+		// \}                     - }
+		pattern = regexp.MustCompile(`(?s)keys\s*\{\s*\"(.*?)\"\s*;\s*(?:\s*\".*?\"\s*;\s*)*\}`)
+
 		keyName := pattern.FindStringSubmatch(match[2])
 		if len(keyName) > 1 {
 			key = getRndcKey(config, keyName[1])
+			if key == "" {
+				log.WithField("key", keyName[1]).Warn("Cannot find key details")
+			}
 		}
 	}
 
@@ -232,15 +243,21 @@ func parseInetSpec(config, excerpt string) (address string, port int64, key stri
 // getCtrlAddressFromBind9Config retrieves the rndc control access address,
 // port, and secret key (if configured) from the configuration `text`.
 //
+// We need to cover the following cases:
+// - no controls clause - BIND9 will open a control socket on localhost
+// - empty controls clause - no control socket will be opened
+// - controls clause with keys - BIND9 will open a control socket
+//
 // Multiple controls clauses may be configured but currently this function
 // only matches the first one.  Multiple access points may be listed inside
 // a single controls clause, but this function currently only matches the
 // first in the list.  A controls clause may look like this:
 //
-//	controls {
-//		inet 127.0.0.1 allow {localhost;};
-//		inet * port 7766 allow {"rndc-users";} keys {"rndc-remote";};
-//	};
+//		controls {
+//			inet 127.0.0.1 allow {localhost;};
+//			inet * port 7766 allow {"rndc-users";};
+//	        keys {"rndc-remote";};
+//		};
 //
 // In this example, "rndc-users" and "rndc-remote" refer to an acl and key
 // clauses.
@@ -253,9 +270,20 @@ func getCtrlAddressFromBind9Config(text string) (controlAddress string, controlP
 	//     controls {
 	//         inet inet_spec [inet_spec] ;
 	//     };
+	// or
+	//     controls { /maybe some whitespace chars here/ };
 	pattern := regexp.MustCompile(`(?s)controls\s*\{\s*(.*)\s*\}\s*;`)
 	controls := pattern.FindStringSubmatch(text)
 	if len(controls) == 0 {
+		log.Debugf("BIND9 has no `controls` clause, assuming defaults (127.0.0.1, port 953)")
+		return "127.0.0.1", 953, ""
+	}
+
+	// See if there's any non-whitespace characters in the controls clause.
+	// If not, there's `controls {};`, which means: disable control socket.
+	txt := strings.TrimSpace(controls[1])
+	if len(txt) == 0 {
+		log.Debugf("BIND9 has rndc support disabled (empty 'controls' found)")
 		return "", 0, ""
 	}
 
@@ -313,7 +341,7 @@ func getStatisticsChannelFromBind9Config(text string) (statsAddress string, stat
 }
 
 // Determine executable using base named directory or system default paths.
-func determineBinPath(baseNamedDir, executable string) (string, error) {
+func determineBinPath(baseNamedDir, executable string, executor storkutil.CommandExecutor) (string, error) {
 	// look for executable in base named directory and sbin or bin subdirectory
 	if baseNamedDir != "" {
 		for _, binDir := range []string{"sbin", "bin"} {
@@ -325,7 +353,7 @@ func determineBinPath(baseNamedDir, executable string) (string, error) {
 	}
 
 	// not found so try to find generally in the system
-	fullPath, err := exec.LookPath(executable)
+	fullPath, err := executor.LookPath(executable)
 	if err != nil {
 		return "", errors.Errorf("cannot determine location of %s", executable)
 	}
@@ -342,15 +370,55 @@ func getPotentialNamedConfLocations() []string {
 	}
 }
 
+// Parses output of the named -V, which contains default paths for named, rndc
+// configurations among many other things. Returns one string:
+// default path for named configuration (may be empty if parsing fails).
+
+// The output of named -V contains the following info we're looking for:
+//
+// default paths:
+//
+//	named configuration:  /etc/bind/named.conf
+//	rndc configuration:   /etc/bind/rndc.conf
+//	DNSSEC root key:      /etc/bind/bind.keys
+//	nsupdate session key: //run/named/session.key
+//	named PID file:       //run/named/named.pid
+//	named lock file:      //run/named/named.lock
+//	geoip-directory:      /usr/share/GeoIP
+func parseNamedDefaultPath(output []byte) string {
+	// Using []byte is inconvenient to use, let's convert to plain string first.
+	text := string(output)
+
+	// Let's use regexp to find interesting string.
+	namedConfPattern := regexp.MustCompile(`named configuration: *(\/.*)`)
+	namedConfMatch := namedConfPattern.FindStringSubmatch(text)
+
+	// If found, Match is an array of strings. match[0] is always full string,
+	// match[1] is the first (and only in this case) group.
+	if len(namedConfMatch) < 2 {
+		log.Warnf("Unable to find 'named configuration:' line in 'named -V' output.")
+		return ""
+	}
+
+	return namedConfMatch[1]
+}
+
 // Detects the running Bind 9 application.
 // It accepts the components of the Bind 9 process name (the "match" argument),
 // the current working directory of the process (the "cwd" argument; it may be
-// empty), and a command executor instance. It uses the arguments to detect the
-// Bind 9 configuration and the Bind 9 communication channels. Returns the
-// collected data or nil if the Bind 9 is not recognized or any error occurs.
+// empty), and a command executor instance. It uses multiple steps to attempt
+// detection:
+//
+// Step 1: Try to parse -c parameter of the running process.
+// Step 2: Checks if STORK_BIND9_CONFIG is defined. If it is, uses that.
+// Step 3: Try to parse output of the named -V command.
+// Step 4: Try to find named.conf in the default locations.
+//
+// Returns the collected data or nil if the Bind 9 is not recognized or any
+// error occurs.
 func detectBind9App(match []string, cwd string, executor storkutil.CommandExecutor) App {
 	if len(match) < 3 {
-		log.Warnf("problem with parsing BIND 9 cmdline: %s", match[0])
+		log.Warnf("Problem with parsing BIND 9 cmdline: %s", match[0])
 		return nil
 	}
 
@@ -362,28 +430,34 @@ func detectBind9App(match []string, cwd string, executor storkutil.CommandExecut
 	// look for config file in cmd params
 	paramsPattern := regexp.MustCompile(`-c\s+(\S+)`)
 	m := paramsPattern.FindStringSubmatch(bind9Params)
+
+	// STEP 1: Let's try to parse -c parameter passed to named.
+	log.Debug("Looking for BIND 9 config file in -c parameter of a running process.")
 	if m != nil {
 		bind9ConfPath = m[1]
 		// if path to config is not absolute then join it with CWD of named
-		if !strings.HasPrefix(bind9ConfPath, "/") {
+		if !path.IsAbs(bind9ConfPath) {
 			bind9ConfPath = path.Join(cwd, bind9ConfPath)
 		}
-	} else {
-		// config path not found in cmdline params so try to guess its location
-		for _, f := range getPotentialNamedConfLocations() {
+		log.Debugf("Found BIND 9 config file in %s based on -c parameter of a running process.", bind9ConfPath)
+	}
+
+	// STEP 2: Check if STORK_BIND9_CONFIG variable is specified it is, we'll use
+	// whatever value is provided. User knows best *cough*.
+	if bind9ConfPath == "" {
+		if f, ok := os.LookupEnv("STORK_BIND9_CONFIG"); ok {
+			log.Debugf("Looking for BIND 9 config in %s as specified in STORK_BIND9_CONFIG variable.", f)
 			if _, err := os.Stat(f); err == nil {
 				bind9ConfPath = f
-				break
+				log.Infof("Found BIND 9 config file in %s, based on STORK_BIND9_CONFIG variable", f)
+			} else {
+				log.Errorf("File specified in STORK_BIND9_CONFIG (%s) not found or unreadable.", f)
 			}
 		}
 	}
 
-	// no config file so nothing to do
-	if bind9ConfPath == "" {
-		log.Warnf("cannot find config file for BIND 9")
-		return nil
-	}
-
+	// STEP 3: If we still don't have anything, let's try to run named -V and
+	// parse its output.
 	// determine config directory
 	bind9ConfDir := path.Dir(bind9ConfPath)
 
@@ -394,15 +468,54 @@ func detectBind9App(match []string, cwd string, executor storkutil.CommandExecut
 		baseNamedDir, _ = filepath.Split(strings.TrimRight(namedDir, "/"))
 	}
 
-	// run named-checkconf on main config file and get preprocessed content of whole config
-	namedCheckconfPath, err := determineBinPath(baseNamedDir, namedCheckconfExec)
-	if err != nil {
-		log.Warnf("cannot find BIND 9 %s: %s", namedCheckconfExec, err)
+	if bind9ConfPath == "" {
+		log.Debugf("Looking for BIND 9 config file in output of `named -V`.")
+		namedPath, err := determineBinPath(baseNamedDir, namedExec, executor)
+		if err != nil {
+			log.Warnf("cannot determine BIND 9 executable %s: %s", namedExec, err)
+			return nil
+		}
+		out, err := executor.Output(namedPath, "-V")
+		if err != nil {
+			log.Warnf("Attempt to run '%s -V' failed. I give up. Error: %s", namedPath, err)
+			return nil
+		}
+		bind9ConfPath = parseNamedDefaultPath(out)
+		if len(bind9ConfPath) > 0 {
+			log.Infof("Found BIND 9 config file in %s based on output of `named -V`.", bind9ConfPath)
+		}
+	}
+
+	// STEP 4: If we still don't have anything, let's look at typical locations.
+	if bind9ConfPath == "" {
+		log.Debugf("Looking for BIND 9 config file in typical locations.")
+		// config path not found in cmdline params so try to guess its location
+		for _, f := range getPotentialNamedConfLocations() {
+			log.Debugf("Looking for BIND 9 config file in %s", f)
+			if _, err := os.Stat(f); err == nil {
+				bind9ConfPath = f
+				log.Infof("Found BIND 9 config file in %s based on typical locations.", bind9ConfPath)
+				break
+			}
+		}
+	}
+
+	// no config file so nothing to do
+	if bind9ConfPath == "" {
+		log.Warnf("Cannot find config file for BIND 9")
 		return nil
 	}
+
+	// run named-checkconf on main config file and get preprocessed content of whole config
+	namedCheckconfPath, err := determineBinPath(baseNamedDir, namedCheckconfExec, executor)
+	if err != nil {
+		log.Warnf("Cannot find BIND 9 %s: %s", namedCheckconfExec, err)
+		return nil
+	}
+
 	out, err := executor.Output(namedCheckconfPath, "-p", bind9ConfPath)
 	if err != nil {
-		log.Warnf("cannot parse BIND 9 config file %s: %+v; %s", bind9ConfPath, err, out)
+		log.Warnf("Cannot parse BIND 9 config file %s: %+v; %s", bind9ConfPath, err, out)
 		return nil
 	}
 	cfgText := string(out)
@@ -410,7 +523,7 @@ func detectBind9App(match []string, cwd string, executor storkutil.CommandExecut
 	// look for control address in config
 	ctrlAddress, ctrlPort, ctrlKey := getCtrlAddressFromBind9Config(cfgText)
 	if ctrlPort == 0 || len(ctrlAddress) == 0 {
-		log.Warnf("found BIND 9 config file (%s) but cannot parse controls clause", bind9ConfPath)
+		log.Warnf("Found BIND 9 config file (%s) but rndc support was disabled (empty `controls` clause)", bind9ConfPath)
 		return nil
 	}
 	accessPoints := []AccessPoint{
@@ -418,6 +531,7 @@ func detectBind9App(match []string, cwd string, executor storkutil.CommandExecut
 			Type:    AccessPointControl,
 			Address: ctrlAddress,
 			Port:    ctrlPort,
+			Key:     ctrlKey,
 		},
 	}
 
@@ -431,7 +545,7 @@ func detectBind9App(match []string, cwd string, executor storkutil.CommandExecut
 			Key:     key,
 		})
 	} else {
-		log.Warnf("cannot parse BIND 9 statistics-channels clause")
+		log.Warnf("Cannot parse BIND 9 statistics-channels clause. Unable to gather statistics.")
 	}
 
 	// rndc is the command to interface with BIND 9.
@@ -442,9 +556,9 @@ func detectBind9App(match []string, cwd string, executor storkutil.CommandExecut
 
 	// determine rndc details
 	rndcClient := NewRndcClient(rndc)
-	err = rndcClient.DetermineDetails(baseNamedDir, bind9ConfDir, ctrlAddress, ctrlPort, ctrlKey)
+	err = rndcClient.DetermineDetails(baseNamedDir, bind9ConfDir, ctrlAddress, ctrlPort, ctrlKey, executor)
 	if err != nil {
-		log.Warnf("cannot determine BIND 9 rndc details: %s", err)
+		log.Warnf("Cannot determine BIND 9 rndc details: %s", err)
 		return nil
 	}
 
