@@ -1,29 +1,22 @@
 package keaconfig
 
 import (
+	_ "embed"
+	"encoding/json"
 	"fmt"
 	"testing"
 
 	require "github.com/stretchr/testify/require"
 )
 
-// Returns test Kea configuration lacking hooks libraries configurations.
-func getTestConfigWithoutHooks(t *testing.T) *Map {
-	configStr := `{
-        "Dhcp4": {
-            "valid-lifetime": 1000
-        }
-    }`
+//go:embed keaconfig_test_dhcp4_all_keys.json
+var allKeysDHCPv4 []byte
 
-	cfg, err := NewFromJSON(configStr)
-	require.NoError(t, err)
-	require.NotNil(t, cfg)
-
-	return cfg
-}
+//go:embed keaconfig_test_dhcp6_all_keys.json
+var allKeysDHCPv6 []byte
 
 // Returns test Kea configuration with empty list of hooks libraries.
-func getTestConfigEmptyHooks(t *testing.T) *Map {
+func getTestConfigEmptyHooks(t *testing.T) *Config {
 	configStr := `{
         "Dhcp4": {
             "valid-lifetime": 1000,
@@ -31,305 +24,104 @@ func getTestConfigEmptyHooks(t *testing.T) *Map {
         }
     }`
 
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
 	return cfg
 }
 
-// Returns test Kea configuration including two hooks libraries.
-func getTestConfigWithHooks(t *testing.T) *Map {
+// Returns test Kea configuration including multiple IPv4 subnets.
+func getTestConfigWithIPv4Subnets(t *testing.T) *Config {
 	configStr := `{
         "Dhcp4": {
-            "hooks-libraries": [
+            "shared-networks": [
                 {
-                    "library": "/usr/lib/kea/libdhcp_lease_cmds.so"
+                    "name": "foo",
+                    "subnet4": [
+                        {
+                            "id": 567,
+                            "subnet": "10.1.0.0/16"
+                        },
+                        {
+                            "id": 678,
+                            "subnet": "10.2.1.0/16"
+                        }
+                    ],
+					"option-data": [
+						{
+							"code": 5,
+							"name": "name-servers",
+							"space": "dhcp4",
+							"data": "192.0.2.1"
+						}
+					]
                 },
                 {
-                    "library": "/usr/lib/kea/libdhcp_ha.so",
-                    "parameters": {
-                        "high-availability": [{
-                            "this-server-name": "server1",
-                            "mode": "load-balancing",
-                            "heartbeat-delay": 10000,
-                            "max-response-delay": 10000,
-                            "max-ack-delay": 5000,
-                            "max-unacked-clients": 5,
-                            "peers": [{
-                                "name": "server1",
-                                "url": "http://192.168.56.33:8000/",
-                                "role": "primary",
-                                "auto-failover": true
-                            }, {
-                                "name": "server2",
-                                "url": "http://192.168.56.66:8000/",
-                                "role": "secondary",
-                                "auto-failover": true
-                            }, {
-                                "name": "server3",
-                                "url": "http://192.168.56.99:8000/",
-                                "role": "backup",
-                                "auto-failover": false
-                            }]
-                        }]
-                    }
+                    "name": "bar",
+                    "subnet4": [
+                        {
+                            "id": 789,
+                            "subnet": "10.3.0.0/16"
+                        },
+                        {
+                            "id": 890,
+                            "subnet": "10.4.0.0/16"
+                        }
+                    ]
                 }
-            ]
-        }
-    }`
-
-	cfg, err := NewFromJSON(configStr)
-	require.NoError(t, err)
-	require.NotNil(t, cfg)
-
-	return cfg
-}
-
-// Returns test HA configuration which lacks some parameters.
-func getTestMinimalHAConfig(t *testing.T) *Map {
-	configStr := `{
-        "Dhcp4": {
-            "hooks-libraries": [
+            ],
+            "subnet4": [
                 {
-                    "library": "/usr/lib/kea/libdhcp_ha.so",
-                    "parameters": {
-                        "high-availability": [{
-                            "this-server-name": "server1",
-                            "mode": "load-balancing",
-                            "heartbeat-delay": 10000
-                        }]
-                    }
+                    "id": 123,
+                    "subnet": "192.0.2.0/24"
+                },
+                {
+                    "id": 234,
+                    "subnet": "192.0.3.0/24"
+                },
+                {
+                    "id": 345,
+                    "subnet": "10.0.0.0/8"
                 }
             ]
         }
     }`
 
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
 	return cfg
 }
 
-// Tests that Logging key is ignored as non-root key.
-func TestGetRootNameNoRoot(t *testing.T) {
-	cfg, err := NewFromJSON(`
-        {
-            "Logging": { }
-        }
-    `)
-	require.NoError(t, err)
-	require.NotNil(t, cfg)
-
-	root, ok := cfg.GetRootName()
-	require.False(t, ok)
-	require.Empty(t, root)
-}
-
-// Tests that new configuration map can be created.
-func TestNewMap(t *testing.T) {
-	cfg := map[string]interface{}{
-		"key": "value",
-	}
-	c := New(&cfg)
-	require.NotNil(t, c)
-	require.Contains(t, *c, "key")
-}
-
-// Tests that the configuration root key can be found.
-func TestGetRootName(t *testing.T) {
-	cfg, err := NewFromJSON(`
-        {
-            "Logging": { },
-            "Dhcp4": {
-                "subnet4": [ ]
-            }
-        }
-    `)
-	require.NoError(t, err)
-	require.NotNil(t, cfg)
-
-	root, ok := cfg.GetRootName()
-	require.True(t, ok)
-	require.Equal(t, "Dhcp4", root)
-}
-
-// Test that a list of configurations of all hooks libraries can be retrieved
-// from the Kea configuration.
-func TestGetHooksLibraries(t *testing.T) {
-	cfg := getTestConfigWithHooks(t)
-
-	libraries := cfg.GetHooksLibraries()
-	require.Len(t, libraries, 2)
-
-	library := libraries[0]
-	require.Equal(t, "/usr/lib/kea/libdhcp_lease_cmds.so", library.Library)
-	require.Empty(t, library.Parameters)
-
-	library = libraries[1]
-	require.Equal(t, "/usr/lib/kea/libdhcp_ha.so", library.Library)
-	require.Contains(t, library.Parameters, "high-availability")
-}
-
-// Test that empty list of hooks is returned when the configuration lacks
-// hooks-libraries parameter.
-func TestGetHooksLibrariesNoHooks(t *testing.T) {
-	cfg := getTestConfigWithoutHooks(t)
-
-	libraries := cfg.GetHooksLibraries()
-	require.Empty(t, libraries)
-}
-
-// Test that configuration of the selected hooks library can be retrieved
-// from the Kea configuration.
-func TestGetHooksLibrary(t *testing.T) {
-	cfg := getTestConfigWithHooks(t)
-
-	path, params, ok := cfg.GetHooksLibrary("libdhcp_ha")
-	require.True(t, ok)
-	require.Equal(t, "/usr/lib/kea/libdhcp_ha.so", path)
-	require.Contains(t, params, "high-availability")
-}
-
-// Test the case when Kea configuration contains empty hooks list and
-// one of the hooks is requested by name.
-func TestGetHooksLibraryEmptyHooks(t *testing.T) {
-	cfg := getTestConfigEmptyHooks(t)
-
-	path, params, ok := cfg.GetHooksLibrary("libdhcp_ha")
-	require.False(t, ok)
-	require.Empty(t, path)
-	require.Empty(t, params)
-}
-
-// Test that the configuration of the HA hooks library can be retrieved
-// and parsed.
-func TestGetHAHooksLibrary(t *testing.T) {
-	cfg := getTestConfigWithHooks(t)
-
-	path, params, ok := cfg.GetHAHooksLibrary()
-	require.True(t, ok)
-
-	require.NotNil(t, params.ThisServerName)
-	require.NotNil(t, params.Mode)
-	require.NotNil(t, params.HeartbeatDelay)
-	require.NotNil(t, params.MaxResponseDelay)
-	require.NotNil(t, params.MaxAckDelay)
-	require.NotNil(t, params.MaxUnackedClients)
-
-	require.Equal(t, "/usr/lib/kea/libdhcp_ha.so", path)
-	require.Equal(t, "server1", *params.ThisServerName)
-	require.Equal(t, "load-balancing", *params.Mode)
-	require.Equal(t, 10000, *params.HeartbeatDelay)
-	require.Equal(t, 10000, *params.MaxResponseDelay)
-	require.Equal(t, 5000, *params.MaxAckDelay)
-	require.Equal(t, 5, *params.MaxUnackedClients)
-	require.Len(t, params.Peers, 3)
-
-	peersFound := make(map[string]bool)
-
-	for _, peer := range params.Peers {
-		require.NotNil(t, peer.Name)
-		require.NotNil(t, peer.URL)
-		require.NotNil(t, peer.Role)
-		require.NotNil(t, peer.AutoFailover)
-		peersFound[*peer.Name] = true
-		switch *peer.Name {
-		case "server1":
-			require.Equal(t, "http://192.168.56.33:8000/", *peer.URL)
-			require.Equal(t, "primary", *peer.Role)
-			require.True(t, *peer.AutoFailover)
-		case "server2":
-			require.Equal(t, "http://192.168.56.66:8000/", *peer.URL)
-			require.Equal(t, "secondary", *peer.Role)
-			require.True(t, *peer.AutoFailover)
-		case "server3":
-			require.Equal(t, "http://192.168.56.99:8000/", *peer.URL)
-			require.Equal(t, "backup", *peer.Role)
-			require.False(t, *peer.AutoFailover)
-		}
-	}
-
-	// There should be three distinct peers found.
-	require.Len(t, peersFound, 3)
-}
-
-// Test that parameters not included in the HA configuration are set
-// to nil.
-func TestGetHAHooksLibraryOptionals(t *testing.T) {
-	cfg := getTestMinimalHAConfig(t)
-
-	_, params, ok := cfg.GetHAHooksLibrary()
-	require.True(t, ok)
-
-	// These parameters were specified.
-	require.NotNil(t, params.ThisServerName)
-	require.NotNil(t, params.Mode)
-	require.NotNil(t, params.HeartbeatDelay)
-
-	// These parameters weren't.
-	require.Nil(t, params.MaxResponseDelay)
-	require.Nil(t, params.MaxAckDelay)
-	require.Nil(t, params.MaxUnackedClients)
-}
-
-// Test the case when Kea configuration contains empty hooks list and
-// HA hooks library is requested.
-func TestGetHAHooksLibraryEmptyHooks(t *testing.T) {
-	cfg := getTestConfigEmptyHooks(t)
-
-	path, params, ok := cfg.GetHAHooksLibrary()
-	require.False(t, ok)
-	require.Empty(t, path)
-	require.Empty(t, params.ThisServerName)
-}
-
-// Checks if the HA peer structure validation works as expected.
-func TestPeerParametersSet(t *testing.T) {
-	p := Peer{}
-	require.False(t, p.IsSet())
-
-	name := "server1"
-	p.Name = &name
-	require.False(t, p.IsSet())
-
-	url := "http://example.org/"
-	p.URL = &url
-	require.False(t, p.IsSet())
-
-	role := "primary"
-	p.Role = &role
-	require.True(t, p.IsSet())
-
-	autoFailover := true
-	p.AutoFailover = &autoFailover
-	require.True(t, p.IsSet())
-}
-
-// Checks if the HA configuration validation works as expected.
-func TestHAConfigParametersSet(t *testing.T) {
-	cfg := HA{}
-
-	require.False(t, cfg.IsSet())
-
-	thisServerName := "server1"
-	cfg.ThisServerName = &thisServerName
-	require.False(t, cfg.IsSet())
-
-	haMode := "load-balancing"
-	cfg.Mode = &haMode
-	require.True(t, cfg.IsSet())
-
-	p := Peer{}
-	cfg.Peers = append(cfg.Peers, p)
-	require.False(t, cfg.IsSet())
-}
-
-// Verifies that a list of loggers is parsed correctly for a daemon.
-func TestGetLoggers(t *testing.T) {
+// Returns a configuration including global DHCP reservations.
+func getTestConfigWithGlobalReservations(t *testing.T, rootName string) *Config {
 	configStr := `{
-        "Dhcp4": {
+		"%s": {
+			"reservations": [
+				{
+					"hw-address": "01:02:03:04:05:06",
+					"hostname": "foo.example.org"
+				},
+				{
+					"duid": "01:01:01:01",
+					"client-classes": ["bar"]
+				}
+			]
+		}
+	}`
+	cfg, err := NewConfig(fmt.Sprintf(configStr, rootName))
+	require.NoError(t, err)
+	require.NotNil(t, cfg)
+
+	return cfg
+}
+
+// Returns a configuration with loggers.
+func getTestConfigWithLoggers(t *testing.T, rootName string) *Config {
+	configStr := `{
+        "%s": {
             "loggers": [
                 {
                     "name": "kea-dhcp4",
@@ -354,9 +146,195 @@ func TestGetLoggers(t *testing.T) {
         }
     }`
 
-	cfg, err := NewFromJSON(configStr)
+	configStr = fmt.Sprintf(configStr, rootName)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
+	return cfg
+}
+
+// Returns test Kea configuration including multiple IPv6 subnets.
+func getTestConfigWithIPv6Subnets(t *testing.T) *Config {
+	configStr := `{
+        "Dhcp6": {
+            "shared-networks": [
+                {
+                    "name": "foo",
+                    "subnet6": [
+                        {
+                            "id": 567,
+                            "subnet": "3000:1::/32"
+                        },
+                        {
+                            "id": 678,
+                            "subnet": "3000:2::/32"
+                        }
+                    ],
+					"option-data": [
+						{
+							"code": 33,
+							"name": "bcmcs-server-dns",
+							"data": "foo.example.org",
+							"space": "dhcp6"
+						}
+					]
+                },
+                {
+                    "name": "bar",
+                    "subnet6": [
+                        {
+                            "id": 789,
+                            "subnet": "3000:3::/32"
+                        },
+                        {
+                            "id": 890,
+                            "subnet": "3000:4::/32"
+                        }
+                    ]
+                }
+            ],
+            "subnet6": [
+                {
+                    "id": 123,
+                    "subnet": "2001:db8:1:0::/64"
+                },
+                {
+                    "id": 234,
+                    "subnet": "2001:db8:2::/64",
+                    "pd-pools": [
+                        {
+                            "prefix": "3000::/16",
+                            "prefix-len": 64,
+                            "delegated-len": 96
+                        }
+                    ]
+                },
+                {
+                    "id": 345,
+                    "subnet": "2001:db8:3::/64"
+                }
+            ]
+        }
+    }`
+
+	cfg, err := NewConfig(configStr)
+	require.NoError(t, err)
+	require.NotNil(t, cfg)
+
+	return cfg
+}
+
+// Test that Kea DHCPv4 configuration is recognised and parsed.
+func TestDecodeDHCPv4(t *testing.T) {
+	var config Config
+	err := json.Unmarshal(allKeysDHCPv4, &config)
+	require.NoError(t, err)
+
+	require.NotNil(t, config.DHCPv4Config)
+
+	marshalled, err := json.Marshal(config)
+	require.NoError(t, err)
+
+	require.JSONEq(t, string(allKeysDHCPv4), string(marshalled))
+}
+
+// Test that Kea DHCPv6 configuration is recognised and parsed.
+func TestDecodeDHCPv6(t *testing.T) {
+	var config Config
+	err := json.Unmarshal(allKeysDHCPv6, &config)
+	require.NoError(t, err)
+
+	require.NotNil(t, config.DHCPv6Config)
+
+	marshalled, err := json.Marshal(config)
+	require.NoError(t, err)
+
+	require.JSONEq(t, string(allKeysDHCPv6), string(marshalled))
+}
+
+// Tests that the configuration can contain comments.
+func TestNewConfigWithComments(t *testing.T) {
+	configStr := `{
+		"Dhcp4": {
+			// A comment.
+		}
+	}`
+	cfg, err := NewConfig(configStr)
+	require.NoError(t, err)
+	require.NotNil(t, cfg)
+}
+
+// Test instantiating a new config from a map.
+func TestNewConfigFromMap(t *testing.T) {
+	cfgMap := &map[string]any{
+		"Dhcp4": map[string]any{
+			"subnet4": []any{
+				map[string]any{
+					"id":     123,
+					"subnet": "192.0.2.0/24",
+				},
+			},
+		},
+	}
+	cfg := NewConfigFromMap(cfgMap)
+	require.NotNil(t, cfg)
+	require.NotNil(t, cfg.Raw)
+	require.NotNil(t, cfg.DHCPv4Config)
+	subnets := cfg.GetSubnets()
+	require.Len(t, subnets, 1)
+	require.EqualValues(t, 123, subnets[0].GetID())
+	require.Equal(t, "192.0.2.0/24", subnets[0].GetPrefix())
+}
+
+// Test that DHCPv4 config is returned as a common data accessor.
+func TestGetCommonConfigAccessorDHCPv4(t *testing.T) {
+	cfg := &Config{
+		DHCPv4Config: &DHCPv4Config{},
+	}
+	require.NotNil(t, cfg)
+	accessor := cfg.getCommonConfigAccessor()
+	require.NotNil(t, accessor)
+	require.Equal(t, cfg.DHCPv4Config, accessor)
+}
+
+// Test that DHCPv6 config is returned as a common data accessor.
+func TestGetCommonConfigAccessorDHCPv6(t *testing.T) {
+	cfg := &Config{
+		DHCPv6Config: &DHCPv6Config{},
+	}
+	require.NotNil(t, cfg)
+	accessor := cfg.getCommonConfigAccessor()
+	require.NotNil(t, accessor)
+	require.Equal(t, cfg.DHCPv6Config, accessor)
+}
+
+// Test that Control Agent config is returned as a common data accessor.
+func TestGetCommonConfigAccessorCtrlAgent(t *testing.T) {
+	cfg := &Config{
+		CtrlAgentConfig: &CtrlAgentConfig{},
+	}
+	require.NotNil(t, cfg)
+	accessor := cfg.getCommonConfigAccessor()
+	require.NotNil(t, accessor)
+	require.Equal(t, cfg.CtrlAgentConfig, accessor)
+}
+
+// Test that D2 config is returned as a common data accessor.
+func TestGetCommonConfigAccessorD2(t *testing.T) {
+	cfg := &Config{
+		D2Config: &D2Config{},
+	}
+	require.NotNil(t, cfg)
+	accessor := cfg.getCommonConfigAccessor()
+	require.NotNil(t, accessor)
+	require.Equal(t, cfg.D2Config, accessor)
+}
+
+// Verifies that a list of loggers is parsed correctly for a daemon.
+func TestGetLoggers(t *testing.T) {
+	cfg := getTestConfigWithLoggers(t, "Dhcp4")
+	require.NotNil(t, cfg)
+	require.NotNil(t, cfg.DHCPv4Config)
 
 	loggers := cfg.GetLoggers()
 	require.Len(t, loggers, 2)
@@ -395,9 +373,10 @@ func TestGetControlSockets(t *testing.T) {
         }
     }`
 
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
+	require.NotNil(t, cfg.CtrlAgentConfig)
 
 	sockets := cfg.GetControlSockets()
 
@@ -443,13 +422,13 @@ func TestConfiguredDaemonNames(t *testing.T) {
         }
     }`
 
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
 	sockets := cfg.GetControlSockets()
 
-	names := sockets.ConfiguredDaemonNames()
+	names := sockets.GetConfiguredDaemonNames()
 	require.Len(t, names, 4)
 
 	require.Contains(t, names, "dhcp4")
@@ -473,14 +452,14 @@ func TestConfiguredDaemonNames(t *testing.T) {
         }
     }`
 
-	cfg, err = NewFromJSON(configStr)
+	cfg, err = NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
 	sockets = cfg.GetControlSockets()
 
 	// This time only two sockets have been configured.
-	names = sockets.ConfiguredDaemonNames()
+	names = sockets.GetConfiguredDaemonNames()
 	require.Len(t, names, 2)
 
 	require.Contains(t, names, "dhcp4")
@@ -547,7 +526,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// All configurations used together.
 	t.Run("all configs present", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, leaseDatabase, hostsDatabase, hostsDatabases, configDatabases, legalConfig)
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -561,7 +540,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// No database configuration.
 	t.Run("no configs present", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, "", "", "", "", "")
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -575,7 +554,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// lease-database
 	t.Run("lease-database only", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, leaseDatabase, "", "", "", "")
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -594,7 +573,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// hosts-database
 	t.Run("hosts-database only", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, "", hostsDatabase, "", "", "")
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -613,7 +592,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// hosts-databases
 	t.Run("hosts-databases only", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, "", "", hostsDatabases, "", "")
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -637,7 +616,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// config-databases
 	t.Run("config-databases only", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, "", "", "", configDatabases, "")
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -661,7 +640,7 @@ func TestGetAllDatabases(t *testing.T) {
 	// legal logging hook
 	t.Run("legal logging only", func(t *testing.T) {
 		configStr := fmt.Sprintf(configTemplate, "", "", "", "", legalConfig)
-		cfg, err := NewFromJSON(configStr)
+		cfg, err := NewConfig(configStr)
 		require.NoError(t, err)
 		require.NotNil(t, cfg)
 
@@ -689,11 +668,11 @@ func TestGetGlobalReservationModesEnableAll(t *testing.T) {
             "reservation-mode": "disabled"
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	// The new settings take precedence over the deprecated
 	// reservation-mode setting.
@@ -719,11 +698,11 @@ func TestGetGlobalReservationModesDisableAll(t *testing.T) {
             "reservation-mode": "out-of-pool"
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	// The new settings take precedence over the deprecated
 	// reservation-mode setting.
@@ -745,11 +724,11 @@ func TestGetGlobalReservationModesDeprecatedDisabled(t *testing.T) {
             "reservation-mode": "disabled"
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	val, set := modes.IsGlobal()
 	require.False(t, val)
@@ -769,11 +748,11 @@ func TestGetGlobalReservationModesDeprecatedGlobal(t *testing.T) {
             "reservation-mode": "global"
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	val, set := modes.IsGlobal()
 	require.True(t, val)
@@ -793,11 +772,11 @@ func TestGetGlobalReservationModesDeprecatedOutOfPool(t *testing.T) {
             "reservation-mode": "out-of-pool"
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	val, set := modes.IsGlobal()
 	require.False(t, val)
@@ -817,11 +796,11 @@ func TestGetGlobalReservationModesDeprecatedAll(t *testing.T) {
             "reservation-mode": "all"
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	val, set := modes.IsGlobal()
 	require.False(t, val)
@@ -840,11 +819,11 @@ func TestGetGlobalReservationModesDefaults(t *testing.T) {
 	configStr := `{
         "Dhcp4": { }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
-	modes := cfg.GetGlobalReservationModes()
+	modes := cfg.GetGlobalReservationParameters()
 	require.NotNil(t, modes)
 	val, set := modes.IsGlobal()
 	require.False(t, val)
@@ -860,32 +839,32 @@ func TestGetGlobalReservationModesDefaults(t *testing.T) {
 // Test a function implementing host reservation mode checking using
 // Kea inheritance scheme.
 func TestIsInAnyReservationModes(t *testing.T) {
-	modes := []ReservationModes{
+	modes := []ReservationParameters{
 		{
-			OutOfPool: nil,
+			ReservationsOutOfPool: nil,
 		},
 		{
-			OutOfPool: new(bool),
+			ReservationsOutOfPool: new(bool),
 		},
 		{
-			OutOfPool: new(bool),
+			ReservationsOutOfPool: new(bool),
 		},
 	}
-	*modes[2].OutOfPool = true
+	*modes[2].ReservationsOutOfPool = true
 
-	require.True(t, IsInAnyReservationModes(func(modes ReservationModes) (bool, bool) {
+	require.True(t, IsInAnyReservationModes(func(modes ReservationParameters) (bool, bool) {
 		return modes.IsOutOfPool()
 	}, modes[0], modes[1], modes[2]))
 
-	require.True(t, IsInAnyReservationModes(func(modes ReservationModes) (bool, bool) {
+	require.True(t, IsInAnyReservationModes(func(modes ReservationParameters) (bool, bool) {
 		return modes.IsOutOfPool()
 	}, modes[0], modes[2], modes[1]))
 
-	require.False(t, IsInAnyReservationModes(func(modes ReservationModes) (bool, bool) {
+	require.False(t, IsInAnyReservationModes(func(modes ReservationParameters) (bool, bool) {
 		return modes.IsOutOfPool()
 	}, modes[1], modes[0]))
 
-	require.False(t, IsInAnyReservationModes(func(modes ReservationModes) (bool, bool) {
+	require.False(t, IsInAnyReservationModes(func(modes ReservationParameters) (bool, bool) {
 		return modes.IsOutOfPool()
 	}, modes[0], modes[0]))
 }
@@ -893,30 +872,29 @@ func TestIsInAnyReservationModes(t *testing.T) {
 // Test that the sensitive data are hidden.
 func TestHideSensitiveData(t *testing.T) {
 	// Arrange
-	input := map[string]interface{}{
-		"foo":      "bar",
+	config, err := NewConfig(`{
+		"foo": "bar",
 		"password": "xxx",
-		"token":    "",
-		"secret":   "aaa",
-		"first": map[string]interface{}{
-			"foo":      "baz",
+		"token": "",
+		"secret": "aaa",
+		"first": {
+			"foo": "baz",
 			"Password": 42,
-			"Token":    nil,
-			"Secret":   "bbb",
-			"second": map[string]interface{}{
-				"foo":      "biz",
+			"Token": null,
+			"Secret": "bbb",
+			"second": {
+				"foo": "biz",
 				"passworD": true,
-				"tokeN":    "yyy",
-				"secreT":   "ccc",
-			},
-		},
-	}
-
-	keaMap := New(&input)
+				"tokeN": "yyy",
+				"secreT": "ccc"
+			}
+		}
+	}`)
+	require.NoError(t, err)
 
 	// Act
-	keaMap.HideSensitiveData()
-	data := *keaMap
+	config.HideSensitiveData()
+	data := config.Raw
 
 	// Assert
 	// Top level
@@ -953,7 +931,7 @@ func TestGetClientClasses(t *testing.T) {
 			]
         }
     }`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
@@ -968,7 +946,7 @@ func TestGetClientClassesNonExisting(t *testing.T) {
 		"Dhcp4": {
 		}
 	}`
-	cfg, err := NewFromJSON(configStr)
+	cfg, err := NewConfig(configStr)
 	require.NoError(t, err)
 	require.NotNil(t, cfg)
 
@@ -976,29 +954,192 @@ func TestGetClientClassesNonExisting(t *testing.T) {
 	require.Empty(t, clientClasses)
 }
 
-// Test that client classes can be deleted from the configuration.
-func TestDeleteClientClasses(t *testing.T) {
+// Test that the subnet ID can be extracted from the Kea configuration for
+// an IPv4 subnet having specified prefix.
+func TestGetLocalIPv4SubnetID(t *testing.T) {
+	cfg := getTestConfigWithIPv4Subnets(t)
+
+	require.EqualValues(t, 567, cfg.GetSubnetByPrefix("10.1.0.0/16").GetID())
+	require.EqualValues(t, 678, cfg.GetSubnetByPrefix("10.2.1.0/16").GetID())
+	require.EqualValues(t, 123, cfg.GetSubnetByPrefix("192.0.2.0/24").GetID())
+	require.EqualValues(t, 234, cfg.GetSubnetByPrefix("192.0.3.0/24").GetID())
+	require.EqualValues(t, 345, cfg.GetSubnetByPrefix("10.0.0.0/8").GetID())
+	require.Nil(t, cfg.GetSubnetByPrefix("10.0.0.0/16"))
+}
+
+// Test that the subnet ID can be extracted from the Kea configuration for
+// an IPv6 subnet having specified prefix.
+func TestGetLocalIPv6SubnetID(t *testing.T) {
+	cfg := getTestConfigWithIPv6Subnets(t)
+
+	require.EqualValues(t, 567, cfg.GetSubnetByPrefix("3000:1::/32").GetID())
+	require.EqualValues(t, 678, cfg.GetSubnetByPrefix("3000:2:0::/32").GetID())
+	require.EqualValues(t, 123, cfg.GetSubnetByPrefix("2001:db8:1::/64").GetID())
+	require.EqualValues(t, 234, cfg.GetSubnetByPrefix("2001:db8:2::/64").GetID())
+	require.EqualValues(t, 345, cfg.GetSubnetByPrefix("2001:db8:3::/64").GetID())
+	require.Nil(t, cfg.GetSubnetByPrefix("2001:db8:4::/64"))
+}
+
+// Test that the top-level multi-threading parameters are returned properly.
+func TestGetMultiThreadingEntry(t *testing.T) {
+	// Arrange
 	configStr := `{
-        "Dhcp4": {
-            "client-classes": [
-				{
-					"name": "foo"
-				},
-				{
-					"name": "bar"
-				}
-			]
-        }
-    }`
-	cfg, err := NewFromJSON(configStr)
+		"Dhcp4": {
+			"multi-threading": {
+			   "enable-multi-threading": true,
+			   "thread-pool-size": 4,
+			   "packet-queue-size": 16
+			}
+		}
+	}`
+
+	config, err := NewConfig(configStr)
 	require.NoError(t, err)
+
+	// Act
+	multiThreading := config.GetMultiThreading()
+
+	// Assert
+	require.NotNil(t, multiThreading)
+	require.NotNil(t, multiThreading.EnableMultiThreading)
+	require.True(t, *multiThreading.EnableMultiThreading)
+	require.NotNil(t, multiThreading.ThreadPoolSize)
+	require.EqualValues(t, 4, *multiThreading.ThreadPoolSize)
+	require.NotNil(t, multiThreading.PacketQueueSize)
+	require.EqualValues(t, 16, *multiThreading.PacketQueueSize)
+}
+
+// Test that the top-level multi-threading structure is returned even if it
+// includes no parameters.
+func TestGetMultiThreadingEntryMissingParameters(t *testing.T) {
+	// Arrange
+	configStr := `{ "Dhcp4": { "multi-threading": { } } }`
+	config, _ := NewConfig(configStr)
+
+	// Act
+	multiThreading := config.GetMultiThreading()
+
+	// Assert
+	require.NotNil(t, multiThreading)
+	require.Nil(t, multiThreading.EnableMultiThreading)
+	require.Nil(t, multiThreading.PacketQueueSize)
+	require.Nil(t, multiThreading.ThreadPoolSize)
+}
+
+// Test that the top-level multi-threading parameters are nil if the
+// multi-threading entry is missing.
+func TestGetMultiThreadingEntryNotExists(t *testing.T) {
+	// Arrange
+	configStr := `{ "Dhcp4": { } }`
+	config, _ := NewConfig(configStr)
+
+	// Act
+	multiThreading := config.GetMultiThreading()
+
+	// Assert
+	require.Nil(t, multiThreading)
+}
+
+// Test getting all shared networks from the DHCPv4 config.
+func TestGetSharedNetworks4(t *testing.T) {
+	cfg := getTestConfigWithIPv4Subnets(t)
 	require.NotNil(t, cfg)
 
-	// Delete client classes multiple times and make sure they
-	// are gone and there is no panic.
-	for i := 0; i < 2; i++ {
-		cfg.DeleteClientClasses()
-		clientClasses := cfg.GetClientClasses()
-		require.Empty(t, clientClasses)
+	sharedNetworks := cfg.GetSharedNetworks(false)
+	require.Len(t, sharedNetworks, 2)
+
+	for i := 0; i < len(sharedNetworks); i++ {
+		require.IsType(t, (*SharedNetwork4)(nil), sharedNetworks[i])
 	}
+	require.Equal(t, "foo", sharedNetworks[0].GetName())
+	require.Len(t, sharedNetworks[0].GetSubnets(), 2)
+	require.Len(t, sharedNetworks[0].GetDHCPOptions(), 1)
+	require.Equal(t, "bar", sharedNetworks[1].GetName())
+	require.Len(t, sharedNetworks[1].GetSubnets(), 2)
+}
+
+// Test getting all shared networks from the DHCPv6 config.
+func TestGetSharedNetworks6(t *testing.T) {
+	cfg := getTestConfigWithIPv6Subnets(t)
+	require.NotNil(t, cfg)
+
+	sharedNetworks := cfg.GetSharedNetworks(false)
+	require.Len(t, sharedNetworks, 2)
+
+	for i := 0; i < len(sharedNetworks); i++ {
+		require.IsType(t, (*SharedNetwork6)(nil), sharedNetworks[i])
+	}
+	require.Equal(t, "foo", sharedNetworks[0].GetName())
+	require.Len(t, sharedNetworks[0].GetSubnets(), 2)
+	require.Len(t, sharedNetworks[0].GetDHCPOptions(), 1)
+	require.Equal(t, "bar", sharedNetworks[1].GetName())
+	require.Len(t, sharedNetworks[1].GetSubnets(), 2)
+	require.Empty(t, sharedNetworks[1].GetDHCPOptions())
+}
+
+// Test getting all top-level IPv4 subnets from the DHCPv4 config.
+func TestGetSubnets4(t *testing.T) {
+	cfg := getTestConfigWithIPv4Subnets(t)
+	require.NotNil(t, cfg)
+
+	subnets := cfg.GetSubnets()
+	require.Len(t, subnets, 3)
+
+	for i := 0; i < len(subnets); i++ {
+		require.IsType(t, (*Subnet4)(nil), subnets[i])
+	}
+	require.EqualValues(t, 123, subnets[0].GetID())
+	require.Equal(t, "192.0.2.0/24", subnets[0].GetPrefix())
+	require.EqualValues(t, 234, subnets[1].GetID())
+	require.Equal(t, "192.0.3.0/24", subnets[1].GetPrefix())
+	require.EqualValues(t, 345, subnets[2].GetID())
+	require.Equal(t, "10.0.0.0/8", subnets[2].GetPrefix())
+}
+
+// Test getting all top-level IPv6 subnets from the DHCPv4 config.
+func TestGetSubnets6(t *testing.T) {
+	cfg := getTestConfigWithIPv6Subnets(t)
+
+	require.NotNil(t, cfg)
+
+	subnets := cfg.GetSubnets()
+	require.Len(t, subnets, 3)
+
+	for i := 0; i < len(subnets); i++ {
+		require.IsType(t, (*Subnet6)(nil), subnets[i])
+	}
+	require.EqualValues(t, 123, subnets[0].GetID())
+	require.Equal(t, "2001:db8:1:0::/64", subnets[0].GetPrefix())
+	require.EqualValues(t, 234, subnets[1].GetID())
+	require.Equal(t, "2001:db8:2::/64", subnets[1].GetPrefix())
+	require.EqualValues(t, 345, subnets[2].GetID())
+	require.Equal(t, "2001:db8:3::/64", subnets[2].GetPrefix())
+}
+
+// Test getting global DHCPv4 host reservations.
+func TestGetReservations4(t *testing.T) {
+	cfg := getTestConfigWithGlobalReservations(t, "Dhcp4")
+	require.NotNil(t, cfg)
+
+	reservations := cfg.GetReservations()
+	require.Len(t, reservations, 2)
+	require.Equal(t, "01:02:03:04:05:06", reservations[0].HWAddress)
+	require.Equal(t, "foo.example.org", reservations[0].Hostname)
+	require.Equal(t, "01:01:01:01", reservations[1].DUID)
+	require.Len(t, reservations[1].ClientClasses, 1)
+	require.Equal(t, "bar", reservations[1].ClientClasses[0])
+}
+
+// Test getting global DHCPv6 host reservations.
+func TestGetReservations6(t *testing.T) {
+	cfg := getTestConfigWithGlobalReservations(t, "Dhcp6")
+	require.NotNil(t, cfg)
+
+	reservations := cfg.GetReservations()
+	require.Len(t, reservations, 2)
+	require.Equal(t, "01:02:03:04:05:06", reservations[0].HWAddress)
+	require.Equal(t, "foo.example.org", reservations[0].Hostname)
+	require.Equal(t, "01:01:01:01", reservations[1].DUID)
+	require.Len(t, reservations[1].ClientClasses, 1)
+	require.Equal(t, "bar", reservations[1].ClientClasses[0])
 }
