@@ -2,13 +2,16 @@ package main
 
 import (
 	"fmt"
-	"net"
 	"os"
+	"reflect"
+	"strconv"
 
+	"github.com/pkg/errors"
 	log "github.com/sirupsen/logrus"
 	"github.com/urfave/cli/v2"
 
 	"isc.org/stork"
+	"isc.org/stork/hooksutil"
 	"isc.org/stork/server/certs"
 	dbops "isc.org/stork/server/database"
 	storkutil "isc.org/stork/util"
@@ -17,97 +20,19 @@ import (
 // Random hash size in the generated password.
 const passwordGenRandomLength = 24
 
-// Establish connection to a database using admin credentials.
-// Specifying db-url is not supported. The maintenance database name,
-// user and password are specified with db-maintenance-name,
-// db-maintenance-user and db-maintenance-password settings.
-func getAdminDBConn(settings *cli.Context) *dbops.PgDB {
-	if !settings.IsSet("db-maintenance-password") {
-		// If password is missing then prompt for it.
-		passwd := storkutil.GetSecretInTerminal("admin password: ")
-		_ = settings.Set("db-maintenance-password", passwd)
-	}
-
-	addrPort := net.JoinHostPort(settings.String("db-host"), settings.String("db-port"))
-
-	// TLS configuration.
-	tlsConfig, err := dbops.GetTLSConfig(settings.String("db-sslmode"),
-		settings.String("db-host"),
-		settings.String("db-sslcert"),
-		settings.String("db-sslkey"),
-		settings.String("db-sslrootcert"))
-	if err != nil {
-		log.Fatal(err.Error())
-	}
-
-	// Use the provided credentials to connect to the database.
-	opts := &dbops.PgOptions{
-		User:      settings.String("db-maintenance-user"),
-		Password:  settings.String("db-maintenance-password"),
-		Database:  settings.String("db-maintenance-name"),
-		Addr:      addrPort,
-		TLSConfig: tlsConfig,
-	}
-
-	db, err := dbops.NewPgDBConn(opts, settings.String("db-trace-queries") != "")
-	if err != nil {
-		log.Fatalf("Unexpected error: %+v", err)
-	}
-
-	// Theoretically, it should not happen but let's make sure in case someone
-	// modifies the NewPgDB function.
-	if db == nil {
-		log.Fatal("Unable to create database instance")
-	}
-	return db
-}
-
 // Establish connection to a database with opts from command line.
-func getDBConn(settings *cli.Context) *dbops.PgDB {
-	var opts *dbops.PgOptions
-	var err error
-
-	dbURL := settings.String("db-url")
-	if dbURL != "" {
-		opts, err = dbops.ParseURL(dbURL)
-		if err != nil {
-			log.Fatalf("Cannot parse database URL: %+v", err)
-		}
-		opts.TLSConfig = nil // ParseURL sets it automatically but we do not use TLS so reset it
-	} else {
-		var passwd string
-		if settings.IsSet("db-password") {
-			passwd = settings.String("db-password")
-		} else {
-			// If password is missing then prompt for it.
-			passwd = storkutil.GetSecretInTerminal("database password: ")
-		}
-
-		addrPort := net.JoinHostPort(settings.String("db-host"), settings.String("db-port"))
-
-		// TLS configuration.
-		tlsConfig, err := dbops.GetTLSConfig(settings.String("db-sslmode"),
-			settings.String("db-host"),
-			settings.String("db-sslcert"),
-			settings.String("db-sslkey"),
-			settings.String("db-sslrootcert"))
-		if err != nil {
-			log.Fatal(err.Error())
-		}
-
-		// Use the provided credentials to connect to the database.
-		opts = &dbops.PgOptions{
-			User:      settings.String("db-user"),
-			Password:  passwd,
-			Database:  settings.String("db-name"),
-			Addr:      addrPort,
-			TLSConfig: tlsConfig,
-		}
+// Returns the database instance. It must be closed by caller.
+func getDBConn(rawFlags *cli.Context) *dbops.PgDB {
+	flags := &dbops.DatabaseCLIFlags{}
+	flags.ReadFromCLI(rawFlags)
+	settings, err := flags.ConvertToDatabaseSettings()
+	if err != nil {
+		log.WithError(err).Fatal("Invalid database settings")
 	}
 
-	db, err := dbops.NewPgDBConn(opts, settings.String("db-trace-queries") != "")
+	db, err := dbops.NewPgDBConn(settings)
 	if err != nil {
-		log.Fatalf("Unexpected error: %+v", err)
+		log.WithError(err).Fatal("Unexpected error")
 	}
 
 	// Theoretically, it should not happen but let's make sure in case someone
@@ -121,50 +46,69 @@ func getDBConn(settings *cli.Context) *dbops.PgDB {
 // Execute db-create command. It prepares new database for the Stork
 // server. It also creates a user that can access this database using
 // a generated or user-specified password and the pgcrypto extension.
-func runDBCreate(settings *cli.Context) {
+func runDBCreate(context *cli.Context) {
+	flags := &dbops.DatabaseCLIFlagsWithMaintenance{}
+	flags.ReadFromCLI(context)
+
 	var err error
 
 	// Prepare logging fields.
 	logFields := log.Fields{
-		"database_name": settings.String("db-name"),
-		"user":          settings.String("db-user"),
+		"database_name": flags.DBName,
+		"user":          flags.User,
 	}
 
 	// Check if the password has been specified explicitly. Otherwise,
 	// generate the password.
-	password := settings.String("db-password")
+	password := flags.Password
 	if len(password) == 0 {
 		password, err = storkutil.Base64Random(passwordGenRandomLength)
 		if err != nil {
-			log.Fatalf("Failed to generate random database password: %s", err)
+			log.WithError(err).Fatal("Failed to generate random database password")
 		}
 		// Only log the password if it has been generated. Otherwise, the
 		// user should know the password.
 		logFields["password"] = password
+		flags.Password = password
 	}
 
 	// Connect to the postgres database using admin credentials.
-	db := getAdminDBConn(settings)
+	settings, err := flags.ConvertToMaintenanceDatabaseSettings()
+	if err != nil {
+		log.WithError(err).Fatal("Invalid database settings")
+	}
+
+	db, err := dbops.NewPgDBConn(settings)
+	if err != nil {
+		log.WithError(err).Fatal("Unexpected error")
+	}
 
 	// Try to create the database and the user with access using
 	// specified password.
-	err = dbops.CreateDatabase(db, settings.String("db-name"), settings.String("db-user"), password, settings.Bool("force"))
+	err = dbops.CreateDatabase(db, flags.DBName, flags.User, flags.Password, context.Bool("force"))
+	// Close the current connection. We will have to connect to the
+	// newly created database instead to create the pgcrypto extension.
+	db.Close()
 	if err != nil {
 		log.Fatalf("%s", err)
 	}
 
-	// Close the current connection. We will have to connect to the
-	// newly created database instead to create the pgcrypto extension.
-	db.Close()
-
 	// Re-use all admin credentials but connect to the new database.
-	_ = settings.Set("db-maintenance-name", settings.String("db-name"))
-	db = getAdminDBConn(settings)
+	settings, err = flags.ConvertToDatabaseSettingsWithMaintenanceCredentials()
+	if err != nil {
+		log.WithError(err).Fatal("Invalid maintenance database settings")
+	}
+
+	db, err = dbops.NewPgDBConn(settings)
+	if err != nil {
+		log.WithError(err).Fatal("Unexpected error")
+	}
+	defer db.Close()
 
 	// Try to create the pgcrypto extension.
 	err = dbops.CreateExtension(db, "pgcrypto")
 	if err != nil {
-		log.Fatalf("%s", err)
+		log.WithError(err).Fatal("Cannot create the database extension")
 	}
 
 	// Database setup successful.
@@ -176,7 +120,7 @@ func runDBCreate(settings *cli.Context) {
 func runDBPasswordGen() {
 	password, err := storkutil.Base64Random(passwordGenRandomLength)
 	if err != nil {
-		log.Fatalf("Failed to generate random database password: %s", err)
+		log.WithError(err).Fatal("Failed to generate random database password")
 	}
 	log.WithFields(log.Fields{
 		"password": password,
@@ -213,7 +157,7 @@ func runDBMigrate(settings *cli.Context, command, version string) {
 	db := getDBConn(settings)
 
 	oldVersion, newVersion, err := dbops.Migrate(db, args...)
-	db.Close()
+	_ = db.Close()
 	if err != nil {
 		log.Fatalf(err.Error())
 	}
@@ -233,6 +177,7 @@ func runDBMigrate(settings *cli.Context, command, version string) {
 // Execute cert export command.
 func runCertExport(settings *cli.Context) error {
 	db := getDBConn(settings)
+	defer db.Close()
 
 	return certs.ExportSecret(db, settings.String("object"), settings.String("file"))
 }
@@ -240,142 +185,121 @@ func runCertExport(settings *cli.Context) error {
 // Execute cert import command.
 func runCertImport(settings *cli.Context) error {
 	db := getDBConn(settings)
+	defer db.Close()
 
 	return certs.ImportSecret(db, settings.String("object"), settings.String("file"))
 }
 
+// Inspect the hook file.
+func inspectHookFile(path string, library *hooksutil.LibraryManager, err error) {
+	if err != nil {
+		log.
+			WithField("file", path).
+			Error(err)
+		return
+	}
+
+	hookProgram, hookVersion, err := library.Version()
+	if err != nil {
+		log.
+			WithField("file", path).
+			Error(err)
+		return
+	}
+
+	log.
+		WithField("file", path).
+		Infof("Hook is compatible with %s@%s", hookProgram, hookVersion)
+}
+
+// Execute inspect hook command.
+func runHookInspect(settings *cli.Context) error {
+	hookPath := settings.String("path")
+	fileInfo, err := os.Stat(hookPath)
+	if err != nil {
+		return errors.Wrapf(err, "cannot stat the hook path: '%s'", hookPath)
+	}
+
+	mode := fileInfo.Mode()
+	switch {
+	case mode.IsDir():
+		return hooksutil.WalkPluginLibraries(hookPath, func(path string, library *hooksutil.LibraryManager, err error) bool {
+			inspectHookFile(path, library, err)
+			return true
+		})
+	case mode.IsRegular():
+		library, err := hooksutil.NewLibraryManager(hookPath)
+		inspectHookFile(hookPath, library, err)
+		return nil
+	default:
+		return errors.Errorf("unsupported file mode: '%s'", mode.String())
+	}
+}
+
+// Parse the general flag definitions into the objects compatible with the CLI library.
+func parseFlagDefinitions(flagDefinitions []*dbops.CLIFlagDefinition) ([]cli.Flag, error) {
+	var flags []cli.Flag
+	for _, definition := range flagDefinitions {
+		var flag cli.Flag
+
+		var aliases []string
+		if definition.Short != "" {
+			aliases = append(aliases, definition.Short)
+		}
+
+		var envVars []string
+		if definition.EnvironmentVariable != "" {
+			envVars = append(envVars, definition.EnvironmentVariable)
+		}
+
+		if definition.Kind == reflect.Int {
+			valueInt, err := strconv.ParseInt(definition.Default, 10, 0)
+			if err != nil {
+				return nil, errors.Wrapf(
+					err, "invalid default value ('%s') for parameter ('%s')",
+					definition.Default, definition.Long,
+				)
+			}
+
+			flag = &cli.Int64Flag{
+				Name:    definition.Long,
+				Aliases: aliases,
+				Usage:   definition.Description,
+				EnvVars: envVars,
+				Value:   valueInt,
+			}
+		} else {
+			flag = &cli.StringFlag{
+				Name:    definition.Long,
+				Aliases: aliases,
+				Usage:   definition.Description,
+				EnvVars: envVars,
+				Value:   definition.Default,
+			}
+		}
+
+		flags = append(flags, flag)
+	}
+
+	return flags, nil
+}
+
 // Prepare urfave cli app with all flags and commands defined.
 func setupApp() *cli.App {
 	cli.VersionPrinter = func(c *cli.Context) {
 		fmt.Println(c.App.Version)
 	}
 
-	dbTLSFlags := []cli.Flag{
-		&cli.StringFlag{
-			Name:    "db-sslmode",
-			Usage:   "The SSL mode for connecting to the database (i.e., disable, require, verify-ca, or verify-full)",
-			Value:   "disable",
-			EnvVars: []string{"STORK_DATABASE_SSLMODE"},
-		},
-		&cli.StringFlag{
-			Name:    "db-sslcert",
-			Usage:   "The location of the SSL certificate used by the server to connect to the database",
-			EnvVars: []string{"STORK_DATABASE_SSLCERT"},
-		},
-		&cli.StringFlag{
-			Name:    "db-sslkey",
-			Usage:   "The location of the SSL key used by the server to connect to the database",
-			EnvVars: []string{"STORK_DATABASE_SSLKEY"},
-		},
-		&cli.StringFlag{
-			Name:    "db-sslrootcert",
-			Usage:   "The location of the root certificate file used to verify the database server's certificate",
-			EnvVars: []string{"STORK_DATABASE_SSLROOTCERT"},
-		},
-		&cli.StringFlag{
-			Name:    "db-trace-queries",
-			Usage:   "Enable tracing SQL queries: \"run\" - only run-time, without migrations, \"all\" - migrations and run-time",
-			Value:   "",
-			EnvVars: []string{"STORK_DATABASE_TRACE_QUERIES"},
-		},
+	dbFlags, err := parseFlagDefinitions((*dbops.DatabaseCLIFlags)(nil).ConvertToCLIFlagDefinitions())
+	if err != nil {
+		log.WithError(err).Fatal("Invalid database CLI flag definitions")
 	}
 
-	dbFlags := []cli.Flag{
-		&cli.StringFlag{
-			Name:    "db-url",
-			Usage:   "The URL to locate the Stork PostgreSQL database",
-			EnvVars: []string{"STORK_DATABASE_URL"},
-		},
-		&cli.StringFlag{
-			Name:    "db-user",
-			Usage:   "The user name for database connections",
-			Aliases: []string{"u"},
-			Value:   "stork",
-			EnvVars: []string{"STORK_DATABASE_USER_NAME"},
-		},
-		&cli.StringFlag{
-			Name:    "db-password",
-			Usage:   "The database password for database connections",
-			EnvVars: []string{"STORK_DATABASE_PASSWORD"},
-		},
-		&cli.StringFlag{
-			Name:    "db-host",
-			Usage:   "The name of the host where the database is available",
-			Value:   "localhost",
-			EnvVars: []string{"STORK_DATABASE_HOST"},
-		},
-		&cli.StringFlag{
-			Name:    "db-port",
-			Usage:   "The port on which the database is available",
-			Aliases: []string{"p"},
-			Value:   "5432",
-			EnvVars: []string{"STORK_DATABASE_PORT"},
-		},
-		&cli.StringFlag{
-			Name:    "db-name",
-			Usage:   "The name of the database to connect to",
-			Aliases: []string{"d"},
-			Value:   "stork",
-			EnvVars: []string{"STORK_DATABASE_NAME"},
-		},
+	dbCreateFlags, err := parseFlagDefinitions((*dbops.DatabaseCLIFlagsWithMaintenance)(nil).ConvertToCLIFlagDefinitions())
+	if err != nil {
+		log.WithError(err).Fatal("Invalid create database CLI flag definitions")
 	}
 
-	dbFlags = append(dbFlags, dbTLSFlags...)
-
-	dbCreateFlags := []cli.Flag{
-		&cli.StringFlag{
-			Name:    "db-maintenance-name",
-			Usage:   "The existing maintenance database name",
-			Aliases: []string{"m"},
-			Value:   "postgres",
-			EnvVars: []string{"STORK_DATABASE_MAINTENANCE_NAME"},
-		},
-		&cli.StringFlag{
-			Name:    "db-maintenance-user",
-			Usage:   "The Postgres database administrator user name",
-			Aliases: []string{"a"},
-			Value:   "postgres",
-			EnvVars: []string{"STORK_DATABASE_MAINTENANCE_USER_NAME"},
-		},
-		&cli.StringFlag{
-			Name:    "db-maintenance-password",
-			Usage:   "The Postgres database administrator password; if not specified, the user will be prompted for the password",
-			EnvVars: []string{"STORK_DATABASE_MAINTENANCE_PASSWORD"},
-		},
-		&cli.StringFlag{
-			Name:    "db-host",
-			Usage:   "The name of the host where the database is available",
-			Value:   "localhost",
-			EnvVars: []string{"STORK_DATABASE_HOST"},
-		},
-		&cli.StringFlag{
-			Name:    "db-port",
-			Usage:   "The port on which the database is available",
-			Aliases: []string{"p"},
-			Value:   "5432",
-			EnvVars: []string{"STORK_DATABASE_PORT"},
-		},
-		&cli.StringFlag{
-			Name:    "db-name",
-			Usage:   "The name of the database to be created",
-			Aliases: []string{"d"},
-			Value:   "stork",
-			EnvVars: []string{"STORK_DATABASE_NAME"},
-		},
-		&cli.StringFlag{
-			Name:    "db-user",
-			Usage:   "The name of the user to be created and granted privileges to the new database",
-			Aliases: []string{"u"},
-			Value:   "stork",
-			EnvVars: []string{"STORK_DATABASE_USER_NAME"},
-		},
-		&cli.StringFlag{
-			Name:  "db-password",
-			Usage: "The user password to the created database; if not specified, a random password is generated",
-		},
-	}
-
-	dbCreateFlags = append(dbCreateFlags, dbTLSFlags...)
 	dbCreateFlags = append(dbCreateFlags, &cli.BoolFlag{
 		Name:    "force",
 		Usage:   "Recreate the database and the user if they exist",
@@ -426,6 +350,16 @@ func setupApp() *cli.App {
 			EnvVars: []string{"STORK_TOOL_CERT_FILE"},
 		})
 
+	hookInspectFlags := []cli.Flag{
+		&cli.StringFlag{
+			Name:     "path",
+			Usage:    "The hook file or directory path",
+			Required: true,
+			Aliases:  []string{"p"},
+			EnvVars:  []string{"STORK_TOOL_HOOK_PATH"},
+		},
+	}
+
 	cli.HelpFlag = &cli.BoolFlag{
 		Name:    "help",
 		Aliases: []string{"h"},
@@ -581,6 +515,14 @@ func setupApp() *cli.App {
 				Category:    "Certificates Management",
 				Action:      runCertImport,
 			},
+			{
+				Name:        "hook-inspect",
+				Usage:       "Prints details about hooks",
+				UsageText:   "stork-tool hook-inspect -p file-or-directory",
+				Description: "",
+				Flags:       hookInspectFlags,
+				Action:      runHookInspect,
+			},
 		},
 	}
 
