@@ -2,6 +2,7 @@ package kea
 
 import (
 	"context"
+	"encoding/json"
 	"testing"
 	"time"
 
@@ -9,6 +10,7 @@ import (
 	"github.com/stretchr/testify/require"
 	keaconfig "isc.org/stork/appcfg/kea"
 	keactrl "isc.org/stork/appctrl/kea"
+	"isc.org/stork/datamodel"
 	"isc.org/stork/server/agentcomm"
 	agentcommtest "isc.org/stork/server/agentcomm/test"
 	appstest "isc.org/stork/server/apps/test"
@@ -76,14 +78,24 @@ func (tm *testManager) scheduleAndGetChange(ctx context.Context, t *testing.T) c
 	require.True(t, ok)
 
 	// The state will be inserted into the database.
-	state, ok := config.GetTransactionState(ctx)
+	state, ok := config.GetTransactionState[ConfigRecipe](ctx)
 	require.True(t, ok)
 
 	// Create the config change entry in the database.
 	scc := &dbmodel.ScheduledConfigChange{
 		DeadlineAt: storkutil.UTCNow().Add(-time.Second * 10),
 		UserID:     userID,
-		Updates:    state.Updates,
+	}
+	for _, u := range state.Updates {
+		update := dbmodel.ConfigUpdate{
+			Target:    u.Target,
+			Operation: u.Operation,
+			DaemonIDs: u.DaemonIDs,
+		}
+		recipe, err := json.Marshal(u.Recipe)
+		require.NoError(t, err)
+		update.Recipe = (*json.RawMessage)(&recipe)
+		scc.Updates = append(scc.Updates, &update)
 	}
 	err := dbmodel.AddScheduledConfigChange(tm.db, scc)
 	require.NoError(t, err)
@@ -95,9 +107,12 @@ func (tm *testManager) scheduleAndGetChange(ctx context.Context, t *testing.T) c
 	change := changes[0]
 
 	// Override the context state.
-	state = config.TransactionState{
+	state = config.TransactionState[ConfigRecipe]{
 		Scheduled: true,
-		Updates:   change.Updates,
+	}
+	for _, u := range change.Updates {
+		update := NewConfigUpdateFromDBModel(u)
+		state.Updates = append(state.Updates, update)
 	}
 	ctx = context.WithValue(ctx, config.StateContextKey, state)
 	return ctx
@@ -129,10 +144,10 @@ func TestBeginHostAdd(t *testing.T) {
 	require.Empty(t, manager.locks)
 
 	// Make sure that the transaction state has been created.
-	state, ok := config.GetTransactionState(ctx)
+	state, ok := config.GetTransactionState[ConfigRecipe](ctx)
 	require.True(t, ok)
 	require.Len(t, state.Updates, 1)
-	require.Equal(t, "kea", state.Updates[0].Target)
+	require.Equal(t, datamodel.AppTypeKea, state.Updates[0].Target)
 	require.Equal(t, "host_add", state.Updates[0].Operation)
 }
 
@@ -146,7 +161,7 @@ func TestApplyHostAdd(t *testing.T) {
 
 	// Transaction state is required because typically it is created by the
 	// BeginHostAdd function.
-	state := config.NewTransactionStateWithUpdate("kea", "host_add")
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_add")
 	ctx := context.WithValue(context.Background(), config.StateContextKey, *state)
 
 	// Simulate submitting new host entry. The host is associated with
@@ -197,7 +212,7 @@ func TestApplyHostAdd(t *testing.T) {
 	require.NoError(t, err)
 
 	// Make sure that the transaction state exists and comprises expected data.
-	returnedState, ok := config.GetTransactionState(ctx)
+	returnedState, ok := config.GetTransactionState[ConfigRecipe](ctx)
 	require.True(t, ok)
 	require.False(t, returnedState.Scheduled)
 
@@ -205,18 +220,16 @@ func TestApplyHostAdd(t *testing.T) {
 	update := state.Updates[0]
 
 	// Basic validation of the retrieved state.
-	require.Equal(t, "kea", update.Target)
+	require.Equal(t, datamodel.AppTypeKea, update.Target)
 	require.Equal(t, "host_add", update.Operation)
 	require.NotNil(t, update.Recipe)
-	require.Contains(t, update.Recipe, "commands")
 
 	// There should be two commands ready to send.
-	commands := update.Recipe["commands"].([]interface{})
+	commands := update.Recipe.Commands
 	require.Len(t, commands, 2)
 
 	// Validate the first command and associated app.
-	command, ok := commands[0].(map[string]interface{})["command"].(*keactrl.Command)
-	require.True(t, ok)
+	command := commands[0].Command
 	marshalled := command.Marshal()
 	require.JSONEq(t,
 		`{
@@ -232,13 +245,11 @@ func TestApplyHostAdd(t *testing.T) {
          }`,
 		marshalled)
 
-	app, ok := commands[0].(map[string]interface{})["app"].(*dbmodel.App)
-	require.True(t, ok)
+	app := commands[0].App
 	require.Equal(t, app, host.LocalHosts[0].Daemon.App)
 
 	// Validate the second command and associated app.
-	command, ok = commands[1].(map[string]interface{})["command"].(*keactrl.Command)
-	require.True(t, ok)
+	command = commands[1].Command
 	marshalled = command.Marshal()
 	require.JSONEq(t,
 		`{
@@ -254,16 +265,21 @@ func TestApplyHostAdd(t *testing.T) {
          }`,
 		marshalled)
 
-	app, ok = commands[1].(map[string]interface{})["app"].(*dbmodel.App)
-	require.True(t, ok)
+	app = commands[1].App
 	require.Equal(t, app, host.LocalHosts[1].Daemon.App)
 }
 
 // Test committing added host, i.e. actually sending control commands to Kea.
 func TestCommitHostAdd(t *testing.T) {
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	_, apps := storktestdbmodel.AddTestHosts(t, db)
+
 	// Create the config manager instance "connected to" fake agents.
 	agents := agentcommtest.NewKeaFakeAgents()
 	manager := newTestManager(&appstest.ManagerAccessorsWrapper{
+		DB:        db,
 		Agents:    agents,
 		DefLookup: dbmodel.NewDHCPOptionDefinitionLookup(),
 	})
@@ -274,12 +290,12 @@ func TestCommitHostAdd(t *testing.T) {
 
 	// Transaction state is required because typically it is created by the
 	// BeginHostAdd function.
-	state := config.NewTransactionStateWithUpdate("kea", "host_add")
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_add")
 	ctx := context.WithValue(context.Background(), config.StateContextKey, *state)
 
 	// Create new host reservation and store it in the context.
 	host := &dbmodel.Host{
-		ID:       1,
+		ID:       1001,
 		Hostname: "cool.example.org",
 		HostIdentifiers: []dbmodel.HostIdentifier{
 			{
@@ -289,7 +305,7 @@ func TestCommitHostAdd(t *testing.T) {
 		},
 		LocalHosts: []dbmodel.LocalHost{
 			{
-				DaemonID: 1,
+				DaemonID: apps[0].Daemons[0].KeaDaemon.DaemonID,
 				Daemon: &dbmodel.Daemon{
 					Name: "dhcp4",
 					App: &dbmodel.App{
@@ -302,9 +318,10 @@ func TestCommitHostAdd(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 			{
-				DaemonID: 2,
+				DaemonID: apps[1].Daemons[0].KeaDaemon.DaemonID,
 				Daemon: &dbmodel.Daemon{
 					Name: "dhcp4",
 					App: &dbmodel.App{
@@ -317,6 +334,7 @@ func TestCommitHostAdd(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 		},
 	}
@@ -350,6 +368,12 @@ func TestCommitHostAdd(t *testing.T) {
              }`,
 			marshalled)
 	}
+
+	// Make sure that the host has been added to the database too.
+	newHost, err := dbmodel.GetHost(db, host.ID)
+	require.NoError(t, err)
+	require.NotNil(t, newHost)
+	require.Len(t, newHost.LocalHosts, 2)
 }
 
 // Test that error is returned when Kea response contains error status code.
@@ -377,7 +401,7 @@ func TestCommitHostAddResponseWithErrorStatus(t *testing.T) {
 
 	// Transaction state is required because typically it is created by the
 	// BeginHostAdd function.
-	state := config.NewTransactionStateWithUpdate("kea", "host_add")
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_add")
 	ctx := context.WithValue(context.Background(), config.StateContextKey, *state)
 
 	// Create new host reservation and store it in the context.
@@ -440,6 +464,8 @@ func TestCommitScheduledHostAdd(t *testing.T) {
 	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
 	defer teardown()
 
+	_, apps := storktestdbmodel.AddTestHosts(t, db)
+
 	agents := agentcommtest.NewKeaFakeAgents()
 	manager := newTestManager(&appstest.ManagerAccessorsWrapper{
 		DB:        db,
@@ -463,7 +489,7 @@ func TestCommitScheduledHostAdd(t *testing.T) {
 
 	// Transaction state is required because typically it is created by the
 	// BeginHostAdd function.
-	state := config.NewTransactionStateWithUpdate("kea", "host_add")
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_add")
 	ctx := context.WithValue(context.Background(), config.StateContextKey, *state)
 
 	// Set user id in the context.
@@ -471,7 +497,7 @@ func TestCommitScheduledHostAdd(t *testing.T) {
 
 	// Create the host and store it in the context.
 	host := &dbmodel.Host{
-		ID: 1,
+		ID: 1001,
 		Subnet: &dbmodel.Subnet{
 			LocalSubnets: []*dbmodel.LocalSubnet{
 				{
@@ -489,7 +515,7 @@ func TestCommitScheduledHostAdd(t *testing.T) {
 		},
 		LocalHosts: []dbmodel.LocalHost{
 			{
-				DaemonID: 1,
+				DaemonID: apps[0].Daemons[0].KeaDaemon.DaemonID,
 				Daemon: &dbmodel.Daemon{
 					Name: "dhcp4",
 					App: &dbmodel.App{
@@ -502,6 +528,7 @@ func TestCommitScheduledHostAdd(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 		},
 	}
@@ -538,6 +565,11 @@ func TestCommitScheduledHostAdd(t *testing.T) {
              }
          }`,
 		marshalled)
+
+	// Make sure that the host has been added to the database.
+	newHost, err := dbmodel.GetHost(db, host.ID)
+	require.NoError(t, err)
+	require.NotNil(t, newHost)
 }
 
 // Test the first stage of updating a host. It checks that the host information
@@ -572,12 +604,12 @@ func TestBeginHostUpdate(t *testing.T) {
 	require.Contains(t, manager.locks, apps[1].Daemons[0].ID)
 
 	// Make sure that the host information has been stored in the context.
-	state, ok := config.GetTransactionState(ctx)
+	state, ok := config.GetTransactionState[ConfigRecipe](ctx)
 	require.True(t, ok)
 	require.Len(t, state.Updates, 1)
-	require.Equal(t, "kea", state.Updates[0].Target)
+	require.Equal(t, datamodel.AppTypeKea, state.Updates[0].Target)
 	require.Equal(t, "host_update", state.Updates[0].Operation)
-	require.Contains(t, state.Updates[0].Recipe, "host_before_update")
+	require.NotNil(t, state.Updates[0].Recipe.HostBeforeUpdate)
 }
 
 // Test second stage of a host update.
@@ -636,8 +668,13 @@ func TestApplyHostUpdate(t *testing.T) {
 	daemonIDs := []int64{1}
 	ctx := context.WithValue(context.Background(), config.DaemonsContextKey, daemonIDs)
 
-	state := config.NewTransactionStateWithUpdate("kea", "host_update", daemonIDs...)
-	err := state.SetValueForUpdate(0, "host_before_update", *host)
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_update", daemonIDs...)
+	recipe := ConfigRecipe{
+		HostConfigRecipeParams: HostConfigRecipeParams{
+			HostBeforeUpdate: host,
+		},
+	}
+	err := state.SetRecipeForUpdate(0, &recipe)
 	require.NoError(t, err)
 	ctx = context.WithValue(ctx, config.StateContextKey, *state)
 
@@ -688,7 +725,7 @@ func TestApplyHostUpdate(t *testing.T) {
 	require.NoError(t, err)
 
 	// Make sure that the transaction state exists and comprises expected data.
-	stateReturned, ok := config.GetTransactionState(ctx)
+	stateReturned, ok := config.GetTransactionState[ConfigRecipe](ctx)
 	require.True(t, ok)
 	require.False(t, stateReturned.Scheduled)
 
@@ -696,21 +733,19 @@ func TestApplyHostUpdate(t *testing.T) {
 	update := stateReturned.Updates[0]
 
 	// Basic validation of the retrieved state.
-	require.Equal(t, "kea", update.Target)
+	require.Equal(t, datamodel.AppTypeKea, update.Target)
 	require.Equal(t, "host_update", update.Operation)
 	require.NotNil(t, update.Recipe)
-	require.Contains(t, update.Recipe, "commands")
-	require.Contains(t, update.Recipe, "host_before_update")
+	require.NotNil(t, update.Recipe.HostBeforeUpdate)
 
 	// There should be four commands ready to send. Two reservation-del and two
 	// reservation-add.
-	commands := update.Recipe["commands"].([]interface{})
+	commands := update.Recipe.Commands
 	require.Len(t, commands, 4)
 
 	// Validate the commands to be sent to Kea.
 	for i := range commands {
-		command, ok := commands[i].(map[string]interface{})["command"].(*keactrl.Command)
-		require.True(t, ok)
+		command := commands[i].Command
 		marshalled := command.Marshal()
 
 		// First are the reservation-del commands sent to respective servers.
@@ -744,17 +779,21 @@ func TestApplyHostUpdate(t *testing.T) {
 				marshalled)
 		}
 		// Verify they are associated with appropriate apps.
-		app, ok := commands[i].(map[string]interface{})["app"].(*dbmodel.App)
-		require.True(t, ok)
+		app := commands[i].App
 		require.Equal(t, app, host.LocalHosts[i%2].Daemon.App)
 	}
 }
 
 // Test committing updated host, i.e. actually sending control commands to Kea.
 func TestCommitHostUpdate(t *testing.T) {
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	_, apps := storktestdbmodel.AddTestHosts(t, db)
+
 	// Create host reservation.
 	host := &dbmodel.Host{
-		ID:       1,
+		ID:       1001,
 		Hostname: "cool.example.org",
 		HostIdentifiers: []dbmodel.HostIdentifier{
 			{
@@ -764,7 +803,7 @@ func TestCommitHostUpdate(t *testing.T) {
 		},
 		LocalHosts: []dbmodel.LocalHost{
 			{
-				DaemonID: 1,
+				DaemonID: apps[0].Daemons[0].KeaDaemon.DaemonID,
 				Daemon: &dbmodel.Daemon{
 					Name: "dhcp4",
 					App: &dbmodel.App{
@@ -777,9 +816,10 @@ func TestCommitHostUpdate(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 			{
-				DaemonID: 2,
+				DaemonID: apps[1].Daemons[0].KeaDaemon.DaemonID,
 				Daemon: &dbmodel.Daemon{
 					Name: "dhcp4",
 					App: &dbmodel.App{
@@ -792,13 +832,17 @@ func TestCommitHostUpdate(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 		},
 	}
 
+	require.NoError(t, dbmodel.AddHost(db, host))
+
 	// Create the config manager instance "connected to" fake agents.
 	agents := agentcommtest.NewKeaFakeAgents()
 	manager := newTestManager(&appstest.ManagerAccessorsWrapper{
+		DB:        db,
 		Agents:    agents,
 		DefLookup: dbmodel.NewDHCPOptionDefinitionLookup(),
 	})
@@ -810,12 +854,25 @@ func TestCommitHostUpdate(t *testing.T) {
 	daemonIDs := []int64{1}
 	ctx := context.WithValue(context.Background(), config.DaemonsContextKey, daemonIDs)
 
-	state := config.NewTransactionStateWithUpdate("kea", "host_update", daemonIDs...)
-	err := state.SetValueForUpdate(0, "host_before_update", *host)
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_update", daemonIDs...)
+	recipe := ConfigRecipe{
+		HostConfigRecipeParams: HostConfigRecipeParams{
+			HostBeforeUpdate: host,
+		},
+	}
+	err := state.SetRecipeForUpdate(0, &recipe)
 	require.NoError(t, err)
 	ctx = context.WithValue(ctx, config.StateContextKey, *state)
 
-	ctx, err = module.ApplyHostUpdate(ctx, host)
+	// Copy the host and modify it. The modifications should be applied in
+	// the database upon commit.
+	modifiedHost := *host
+	modifiedHost.CreatedAt = time.Time{}
+	modifiedHost.Hostname = "modified.example.org"
+	modifiedHost.LocalHosts[0].NextServer = "192.0.2.22"
+	modifiedHost.LocalHosts[1].NextServer = "192.0.2.22"
+
+	ctx, err = module.ApplyHostUpdate(ctx, &modifiedHost)
 	require.NoError(t, err)
 
 	// Committing the host should result in sending control commands to Kea servers.
@@ -859,13 +916,23 @@ func TestCommitHostUpdate(t *testing.T) {
                          "reservation": {
                              "subnet-id": 0,
                              "hw-address": "010203040506",
-                             "hostname": "cool.example.org"
+                             "hostname": "modified.example.org",
+							 "next-server": "192.0.2.22"
                          }
                      }
                  }`,
 				marshalled)
 		}
 	}
+
+	// Make sure that the host has been updated in the database.
+	updatedHost, err := dbmodel.GetHost(db, host.ID)
+	require.NoError(t, err)
+	require.NotNil(t, updatedHost)
+	require.Equal(t, "modified.example.org", updatedHost.Hostname)
+	require.Len(t, updatedHost.LocalHosts, 2)
+	require.Equal(t, "192.0.2.22", updatedHost.LocalHosts[0].NextServer)
+	require.Equal(t, "192.0.2.22", updatedHost.LocalHosts[0].NextServer)
 }
 
 // Test that error is returned when Kea response contains error status code.
@@ -923,8 +990,13 @@ func TestCommitHostUpdateResponseWithErrorStatus(t *testing.T) {
 	daemonIDs := []int64{1}
 	ctx := context.WithValue(context.Background(), config.DaemonsContextKey, daemonIDs)
 
-	state := config.NewTransactionStateWithUpdate("kea", "host_update", daemonIDs...)
-	err := state.SetValueForUpdate(0, "host_before_update", *host)
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_update", daemonIDs...)
+	recipe := ConfigRecipe{
+		HostConfigRecipeParams: HostConfigRecipeParams{
+			HostBeforeUpdate: host,
+		},
+	}
+	err := state.SetRecipeForUpdate(0, &recipe)
 	require.NoError(t, err)
 	ctx = context.WithValue(ctx, config.StateContextKey, *state)
 
@@ -940,9 +1012,14 @@ func TestCommitHostUpdateResponseWithErrorStatus(t *testing.T) {
 
 // Test scheduling config changes in the database, retrieving and committing it.
 func TestCommitScheduledHostUpdate(t *testing.T) {
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	_, apps := storktestdbmodel.AddTestHosts(t, db)
+
 	// Create the host.
 	host := &dbmodel.Host{
-		ID: 1,
+		ID: 1001,
 		Subnet: &dbmodel.Subnet{
 			LocalSubnets: []*dbmodel.LocalSubnet{
 				{
@@ -960,7 +1037,7 @@ func TestCommitScheduledHostUpdate(t *testing.T) {
 		},
 		LocalHosts: []dbmodel.LocalHost{
 			{
-				DaemonID: 1,
+				DaemonID: apps[0].Daemons[0].KeaDaemon.DaemonID,
 				Daemon: &dbmodel.Daemon{
 					Name: "dhcp4",
 					App: &dbmodel.App{
@@ -973,12 +1050,11 @@ func TestCommitScheduledHostUpdate(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 		},
 	}
-
-	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
-	defer teardown()
+	require.NoError(t, dbmodel.AddHost(db, host))
 
 	agents := agentcommtest.NewKeaFakeAgents()
 	manager := newTestManager(&appstest.ManagerAccessorsWrapper{
@@ -1006,12 +1082,22 @@ func TestCommitScheduledHostUpdate(t *testing.T) {
 	ctx := context.WithValue(context.Background(), config.DaemonsContextKey, daemonIDs)
 	ctx = context.WithValue(ctx, config.UserContextKey, int64(user.ID))
 
-	state := config.NewTransactionStateWithUpdate("kea", "host_update", daemonIDs...)
-	err = state.SetValueForUpdate(0, "host_before_update", *host)
+	state := config.NewTransactionStateWithUpdate[ConfigRecipe](datamodel.AppTypeKea, "host_update", daemonIDs...)
+	recipe := ConfigRecipe{
+		HostConfigRecipeParams: HostConfigRecipeParams{
+			HostBeforeUpdate: host,
+		},
+	}
+	err = state.SetRecipeForUpdate(0, &recipe)
 	require.NoError(t, err)
 	ctx = context.WithValue(ctx, config.StateContextKey, *state)
 
-	ctx, err = module.ApplyHostUpdate(ctx, host)
+	// Copy the host and modify it. The modifications should be applied in
+	// the database upon commit.
+	modifiedHost := *host
+	modifiedHost.Hostname = "modified.example.org"
+
+	ctx, err = module.ApplyHostUpdate(ctx, &modifiedHost)
 	require.NoError(t, err)
 
 	// Simulate scheduling the config change and retrieving it from the database.
@@ -1055,13 +1141,18 @@ func TestCommitScheduledHostUpdate(t *testing.T) {
                          "reservation": {
                              "subnet-id": 123,
                              "hw-address": "010203040506",
-                             "hostname": "cool.example.org"
+                             "hostname": "modified.example.org"
                          }
                      }
                  }`,
 				marshalled)
 		}
 	}
+	// Make sure that the host has been added to the database too.
+	updatedHost, err := dbmodel.GetHost(db, host.ID)
+	require.NoError(t, err)
+	require.NotNil(t, updatedHost)
+	require.Equal(t, updatedHost.Hostname, "modified.example.org")
 }
 
 // Test first stage of deleting a host.
@@ -1109,6 +1200,7 @@ func TestApplyHostDelete(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 			{
 				DaemonID: 2,
@@ -1124,6 +1216,7 @@ func TestApplyHostDelete(t *testing.T) {
 						},
 					},
 				},
+				DataSource: dbmodel.HostDataSourceAPI,
 			},
 		},
 	}
@@ -1131,7 +1224,7 @@ func TestApplyHostDelete(t *testing.T) {
 	require.NoError(t, err)
 
 	// Make sure that the transaction state exists and comprises expected data.
-	state, ok := config.GetTransactionState(ctx)
+	state, ok := config.GetTransactionState[ConfigRecipe](ctx)
 	require.True(t, ok)
 	require.False(t, state.Scheduled)
 
@@ -1139,18 +1232,16 @@ func TestApplyHostDelete(t *testing.T) {
 	update := state.Updates[0]
 
 	// Basic validation of the retrieved state.
-	require.Equal(t, "kea", update.Target)
+	require.Equal(t, datamodel.AppTypeKea, update.Target)
 	require.Equal(t, "host_delete", update.Operation)
 	require.NotNil(t, update.Recipe)
-	require.Contains(t, update.Recipe, "commands")
 
 	// There should be two commands ready to send.
-	commands := update.Recipe["commands"].([]interface{})
+	commands := update.Recipe.Commands
 	require.Len(t, commands, 2)
 
 	// Validate the first command and associated app.
-	command, ok := commands[0].(map[string]interface{})["command"].(*keactrl.Command)
-	require.True(t, ok)
+	command := commands[0].Command
 	marshalled := command.Marshal()
 	require.JSONEq(t,
 		`{
@@ -1164,13 +1255,11 @@ func TestApplyHostDelete(t *testing.T) {
          }`,
 		marshalled)
 
-	app, ok := commands[0].(map[string]interface{})["app"].(*dbmodel.App)
-	require.True(t, ok)
+	app := commands[0].App
 	require.Equal(t, app, host.LocalHosts[0].Daemon.App)
 
 	// Validate the second command and associated app.
-	command, ok = commands[1].(map[string]interface{})["command"].(*keactrl.Command)
-	require.True(t, ok)
+	command = commands[1].Command
 	marshalled = command.Marshal()
 	require.JSONEq(t,
 		`{
@@ -1184,8 +1273,7 @@ func TestApplyHostDelete(t *testing.T) {
          }`,
 		marshalled)
 
-	app, ok = commands[1].(map[string]interface{})["app"].(*dbmodel.App)
-	require.True(t, ok)
+	app = commands[1].App
 	require.Equal(t, app, host.LocalHosts[1].Daemon.App)
 }
 
