@@ -11,22 +11,23 @@ import (
 
 // Part of machine table in database that describes state of machine. In DB it is stored as JSONB.
 type MachineState struct {
-	AgentVersion         string
-	Cpus                 int64
-	CpusLoad             string
-	Memory               int64
-	Hostname             string
-	Uptime               int64
-	UsedMemory           int64
-	Os                   string
-	Platform             string
-	PlatformFamily       string
-	PlatformVersion      string
-	KernelVersion        string
-	KernelArch           string
-	VirtualizationSystem string
-	VirtualizationRole   string
-	HostID               string
+	AgentVersion             string
+	Cpus                     int64
+	CpusLoad                 string
+	Memory                   int64
+	Hostname                 string
+	Uptime                   int64
+	UsedMemory               int64
+	Os                       string
+	Platform                 string
+	PlatformFamily           string
+	PlatformVersion          string
+	KernelVersion            string
+	KernelArch               string
+	VirtualizationSystem     string
+	VirtualizationRole       string
+	HostID                   string
+	AgentUsesHTTPCredentials bool
 }
 
 // Represents a machine held in machine table in the database.
@@ -104,6 +105,30 @@ func GetMachineByAddressAndAgentPort(db *pg.DB, address string, agentPort int64)
 	return &machine, nil
 }
 
+// Get a machine by the machine address and the access point port.
+// Optionally, it filters access points by type.
+func GetMachineByAddressAndAccessPointPort(db *pg.DB, machineAddress string, accessPointPort int64, accessPointType *string) (*Machine, error) {
+	machine := Machine{}
+	q := db.Model(&machine).
+		Relation(string(MachineRelationAppAccessPoints)).
+		Relation(string(MachineRelationDaemons)).
+		Join("JOIN access_point").JoinOn("machine.id = access_point.machine_id").
+		Where("machine.address = ?", machineAddress).
+		Where("access_point.port = ?", accessPointPort)
+
+	if accessPointType != nil {
+		q = q.Where("access_point.type = ?", accessPointType)
+	}
+
+	err := q.Select()
+	if errors.Is(err, pg.ErrNoRows) {
+		return nil, nil
+	} else if err != nil {
+		return nil, pkgerrors.Wrapf(err, "problem getting machine by the '%s' machine address and the '%d' access point port", machineAddress, accessPointPort)
+	}
+	return &machine, nil
+}
+
 // Get a machine by its ID with default relations.
 func GetMachineByID(db *pg.DB, id int64) (*Machine, error) {
 	return GetMachineByIDWithRelations(db, id,
