@@ -20,7 +20,7 @@ func init() {
 // generic information about the service such as ID, service name
 // and service type.
 type BaseService struct {
-	tableName   struct{} `pg:"service"` //nolint:unused // ,structcheck // ToDo: restore nolint declaration. Structchecks temporary doesn't support Go 1.18.
+	tableName   struct{} `pg:"service"` //nolint:unused
 	ID          int64
 	Name        string
 	ServiceType string
@@ -75,7 +75,7 @@ const (
 // High Availability specific information. It is embedded in the
 // Service structure.
 type BaseHAService struct {
-	tableName                   struct{} `pg:"ha_service"` //nolint:unused // ,structcheck // ToDo: restore nolint declaration. Structchecks temporary doesn't support Go 1.18.
+	tableName                   struct{} `pg:"ha_service"` //nolint:unused
 	ID                          int64
 	ServiceID                   int64
 	HAType                      HAType
@@ -202,7 +202,7 @@ func addService(tx *pg.Tx, service *Service) error {
 	// Insert generic information into the service table.
 	_, err := tx.Model(service).Insert()
 	if err != nil {
-		err = pkgerrors.Wrapf(err, "Problem adding new service")
+		err = pkgerrors.Wrapf(err, "problem adding new service")
 		return err
 	}
 
