@@ -14,7 +14,9 @@ import (
 	"github.com/go-pg/pg/v10"
 	"github.com/stretchr/testify/require"
 	keaconfig "isc.org/stork/appcfg/kea"
+	dhcpmodel "isc.org/stork/datamodel/dhcp"
 	dbtest "isc.org/stork/server/database/test"
+	storkutil "isc.org/stork/util"
 )
 
 // Simple mock for utilizationStatistics for testing purposes.
@@ -691,6 +693,16 @@ func TestCommitNetworksIntoDB(t *testing.T) {
 							},
 						},
 					},
+					LocalSubnets: []*LocalSubnet{
+						{
+							LocalSubnetID: 13,
+						},
+					},
+				},
+			},
+			LocalSharedNetworks: []*LocalSharedNetwork{
+				{
+					DHCPOptionSetHash: "xyz",
 				},
 			},
 		},
@@ -719,6 +731,11 @@ func TestCommitNetworksIntoDB(t *testing.T) {
 					},
 				},
 			},
+			LocalSubnets: []*LocalSubnet{
+				{
+					SubnetID: 14,
+				},
+			},
 		},
 	}
 	// Attempt to create the global shared network and subnet.
@@ -730,6 +747,14 @@ func TestCommitNetworksIntoDB(t *testing.T) {
 	returnedSubnets, err := GetAllSubnets(db, 0)
 	require.NoError(t, err)
 	require.Len(t, returnedSubnets, 2)
+	require.Len(t, returnedSubnets[0].LocalSubnets, 1)
+	require.Len(t, returnedSubnets[1].LocalSubnets, 1)
+
+	// There should be one shared network.
+	returnedNetworks, err := GetAllSharedNetworks(db, 4)
+	require.NoError(t, err)
+	require.Len(t, returnedNetworks, 1)
+	require.Len(t, returnedNetworks[0].LocalSharedNetworks, 1)
 
 	returnedHosts, err := GetHostsBySubnetID(db, returnedSubnets[0].ID)
 	require.NoError(t, err)
@@ -1202,76 +1227,61 @@ func BenchmarkAddSubnet(b *testing.B) {
 // one checking performance of the function with indexing and without indexing.
 // The function execution time should be significantly longer without indexing.
 func BenchmarkAddDaemonToSubnet(b *testing.B) {
-	testCases := []string{"without indexing", "with indexing"}
+	db, _, teardown := dbtest.SetupDatabaseTestCase(b)
+	defer teardown()
 
-	// Run sub tests.
-	for _, testCase := range testCases {
-		tc := testCase
-		b.Run(tc, func(b *testing.B) {
-			db, _, teardown := dbtest.SetupDatabaseTestCase(b)
-			defer teardown()
+	tx, _ := db.Begin()
 
-			tx, _ := db.Begin()
+	// Add many subnets to the database.
+	subnets := []Subnet{}
+	keaSubnets := []interface{}{}
+	for i := 0; i < 10000; i++ {
+		prefix := fmt.Sprintf("%d.%d.%d.", uint8(i>>16), uint8(i>>8), uint8(i))
+		subnet := Subnet{
+			Prefix: prefix + "0/24",
+		}
+		keaSubnet := map[string]interface{}{
+			"id":     i + 1,
+			"subnet": prefix + "0/24",
+		}
+		AddSubnet(tx, &subnet)
+		subnets = append(subnets, subnet)
+		keaSubnets = append(keaSubnets, keaSubnet)
+	}
+	tx.Commit()
 
-			// Add many subnets to the database.
-			subnets := []Subnet{}
-			keaSubnets := []interface{}{}
-			for i := 0; i < 10000; i++ {
-				prefix := fmt.Sprintf("%d.%d.%d.", uint8(i>>16), uint8(i>>8), uint8(i))
-				subnet := Subnet{
-					Prefix: prefix + "0/24",
-				}
-				keaSubnet := map[string]interface{}{
-					"id":     i + 1,
-					"subnet": prefix + "0/24",
-				}
-				AddSubnet(tx, &subnet)
-				subnets = append(subnets, subnet)
-				keaSubnets = append(keaSubnets, keaSubnet)
-			}
-			tx.Commit()
+	// Also create the configuration including these subnets for the app.
+	rawConfig := &map[string]interface{}{
+		"Dhcp4": map[string]interface{}{
+			"subnet4": keaSubnets,
+		},
+	}
+	daemon := NewKeaDaemon("dhcp4", true)
+	daemon.SetConfig(NewKeaConfig(rawConfig))
 
-			// Also create the configuration including these subnets for the app.
-			rawConfig := &map[string]interface{}{
-				"Dhcp4": map[string]interface{}{
-					"subnet4": keaSubnets,
-				},
-			}
-			daemon := NewKeaDaemon("dhcp4", true)
-			daemon.SetConfig(NewKeaConfig(rawConfig))
+	// Add machine/app.
+	machine := &Machine{
+		ID:        0,
+		Address:   "localhost",
+		AgentPort: 8080,
+	}
+	AddMachine(db, machine)
+	app := &App{
+		ID:        0,
+		Type:      AppTypeKea,
+		MachineID: machine.ID,
+		Daemons: []*Daemon{
+			daemon,
+		},
+	}
+	AddApp(db, app)
 
-			// When measuring time with indexing, we need to build indexes before
-			// running the actual benchmark.
-			if tc == "with indexing" {
-				indexedSubnets := keaconfig.NewIndexedSubnets(daemon.KeaDaemon.Config)
-				daemon.KeaDaemon.KeaDHCPDaemon.IndexedSubnets = indexedSubnets
-			}
-
-			// Add machine/app.
-			machine := &Machine{
-				ID:        0,
-				Address:   "localhost",
-				AgentPort: 8080,
-			}
-			AddMachine(db, machine)
-			app := &App{
-				ID:        0,
-				Type:      AppTypeKea,
-				MachineID: machine.ID,
-				Daemons: []*Daemon{
-					daemon,
-				},
-			}
-			AddApp(db, app)
-
-			// Run the actual benchmark.
-			rand.Seed(time.Now().UTC().UnixNano())
-			b.ResetTimer()
-			for i := 0; i < b.N; i++ {
-				subnetIndex := rand.Intn(len(subnets))
-				AddDaemonToSubnet(db, &subnets[subnetIndex], app.Daemons[0])
-			}
-		})
+	// Run the actual benchmark.
+	rand.Seed(time.Now().UTC().UnixNano())
+	b.ResetTimer()
+	for i := 0; i < b.N; i++ {
+		subnetIndex := rand.Intn(len(subnets))
+		AddDaemonToSubnet(db, &subnets[subnetIndex], app.Daemons[0])
 	}
 }
 
@@ -1298,3 +1308,208 @@ func TestSubnetsByPageFiltersSetIPv6Family(t *testing.T) {
 	// Assert
 	require.EqualValues(t, 6, *filters.Family)
 }
+
+// Test implementation of the keaconfig.Subnet interface (GetID() function).
+func TestSubnetGetID(t *testing.T) {
+	subnet := Subnet{
+		LocalSubnets: []*LocalSubnet{
+			{
+				SubnetID: 10,
+				DaemonID: 110,
+			},
+			{
+				SubnetID: 11,
+				DaemonID: 111,
+			},
+		},
+	}
+	require.EqualValues(t, 10, subnet.GetID(110))
+	require.EqualValues(t, 11, subnet.GetID(111))
+	require.Zero(t, subnet.GetID(1000))
+}
+
+// Test implementation of the keaconfig.Subnet interface (GetKeaParameters()
+// function).
+func TestSubnetGetKeaParameters(t *testing.T) {
+	subnet := Subnet{
+		LocalSubnets: []*LocalSubnet{
+			{
+				DaemonID: 110,
+				KeaParameters: &keaconfig.SubnetParameters{
+					Allocator: storkutil.Ptr("random"),
+				},
+			},
+			{
+				DaemonID: 111,
+				KeaParameters: &keaconfig.SubnetParameters{
+					Allocator: storkutil.Ptr("iterative"),
+				},
+			},
+		},
+	}
+	params0 := subnet.GetKeaParameters(110)
+	require.NotNil(t, params0)
+	require.Equal(t, "random", *params0.Allocator)
+	params1 := subnet.GetKeaParameters(111)
+	require.NotNil(t, params1)
+	require.Equal(t, "iterative", *params1.Allocator)
+
+	require.Nil(t, subnet.GetKeaParameters(1000))
+}
+
+// Test implementation of the dhcpmodel.SubnetAccessor interface (GetPrefix() function).
+func TestSubnetGetPrefix(t *testing.T) {
+	subnet := Subnet{
+		Prefix: "3001::/64",
+	}
+	require.Equal(t, "3001::/64", subnet.GetPrefix())
+}
+
+// Test implementation of the dhcpmodel.SubnetAccessor interface (GetAddressPools() function).
+func TestSubnetGetAddressPools(t *testing.T) {
+	subnet := Subnet{
+		AddressPools: []AddressPool{
+			{
+				LowerBound: "192.0.2.1",
+				UpperBound: "192.0.2.10",
+			},
+			{
+				LowerBound: "192.0.2.20",
+				UpperBound: "192.0.2.30",
+			},
+		},
+	}
+	pools := subnet.GetAddressPools()
+	require.Len(t, pools, 2)
+	require.Equal(t, "192.0.2.1", pools[0].GetLowerBound())
+	require.Equal(t, "192.0.2.10", pools[0].GetUpperBound())
+	require.Equal(t, "192.0.2.20", pools[1].GetLowerBound())
+	require.Equal(t, "192.0.2.30", pools[1].GetUpperBound())
+}
+
+// Test implementation of the dhcpmodel.SubnetAccessor interface (GetPrefixPools() function).
+func TestSubnetGetPrefixPools(t *testing.T) {
+	subnet := Subnet{
+		PrefixPools: []PrefixPool{
+			{
+				Prefix:       "2001:db8:1:1::/64",
+				DelegatedLen: 80,
+			},
+			{
+				Prefix:       "2001:db8:1:2::/64",
+				DelegatedLen: 80,
+			},
+		},
+	}
+	pools := subnet.GetPrefixPools()
+	require.Len(t, pools, 2)
+	require.Equal(t, "2001:db8:1:1::/64", pools[0].GetModel().Prefix)
+	require.EqualValues(t, 80, pools[0].GetModel().DelegatedLen)
+	require.Equal(t, "2001:db8:1:2::/64", pools[1].GetModel().Prefix)
+	require.EqualValues(t, 80, pools[1].GetModel().DelegatedLen)
+}
+
+// Test implementation of the dhcpmodel.SubnetAccessor interface (GetDHCPOptions() function).
+func TestSubnetGetDHCPOptions(t *testing.T) {
+	subnet := Subnet{
+		LocalSubnets: []*LocalSubnet{
+			{
+				DaemonID: 110,
+				DHCPOptionSet: []DHCPOption{
+					{
+						Code:  7,
+						Space: dhcpmodel.DHCPv4OptionSpace,
+					},
+				},
+			},
+			{
+				DaemonID: 111,
+				DHCPOptionSet: []DHCPOption{
+					{
+						Code:  8,
+						Space: dhcpmodel.DHCPv4OptionSpace,
+					},
+				},
+			},
+		},
+	}
+	options0 := subnet.GetDHCPOptions(110)
+	require.Len(t, options0, 1)
+	require.EqualValues(t, 7, options0[0].GetCode())
+
+	options1 := subnet.GetDHCPOptions(111)
+	require.Len(t, options1, 1)
+	require.EqualValues(t, 8, options1[0].GetCode())
+
+	require.Nil(t, subnet.GetDHCPOptions(1000))
+}
+
+// Test that LocalSubnet instance is appended to the Subnet when there is
+// no corresponding LocalSubnet, and it is replaced when the corresponding
+// LocalSubnet exists.
+func TestLocalSubnet(t *testing.T) {
+	// Create a subnet with one local subnet.
+	subnet := Subnet{
+		ID: 123,
+		LocalSubnets: []*LocalSubnet{
+			{
+				DaemonID: 1,
+				KeaParameters: &keaconfig.SubnetParameters{
+					Allocator: storkutil.Ptr("random"),
+				},
+			},
+		},
+	}
+	// Create another local shared network and ensure there are now two.
+	subnet.SetLocalSubnet(&LocalSubnet{
+		DaemonID: 2,
+	})
+	require.Len(t, subnet.LocalSubnets, 2)
+	require.EqualValues(t, 1, subnet.LocalSubnets[0].DaemonID)
+	require.EqualValues(t, 2, subnet.LocalSubnets[1].DaemonID)
+
+	// Replace the first instance with a new one.
+	subnet.SetLocalSubnet(&LocalSubnet{
+		DaemonID: 1,
+		KeaParameters: &keaconfig.SubnetParameters{
+			Allocator: storkutil.Ptr("iterative"),
+		},
+	})
+	require.Len(t, subnet.LocalSubnets, 2)
+	require.EqualValues(t, 1, subnet.LocalSubnets[0].DaemonID)
+	require.EqualValues(t, 2, subnet.LocalSubnets[1].DaemonID)
+	require.NotNil(t, subnet.LocalSubnets[0].KeaParameters)
+	require.Equal(t, "iterative", *subnet.LocalSubnets[0].KeaParameters.Allocator)
+}
+
+// Test that LocalSubnets between two Subnet instances can be combined in a
+// single instance.
+func TestJoinSubnets(t *testing.T) {
+	subnet0 := Subnet{
+		ID: 1,
+		LocalSubnets: []*LocalSubnet{
+			{
+				DaemonID: 1,
+			},
+			{
+				DaemonID: 2,
+			},
+		},
+	}
+	subnet1 := Subnet{
+		ID: 1,
+		LocalSubnets: []*LocalSubnet{
+			{
+				DaemonID: 2,
+			},
+			{
+				DaemonID: 3,
+			},
+		},
+	}
+	subnet0.Join(&subnet1)
+	require.Len(t, subnet0.LocalSubnets, 3)
+	require.EqualValues(t, 1, subnet0.LocalSubnets[0].DaemonID)
+	require.EqualValues(t, 2, subnet0.LocalSubnets[1].DaemonID)
+	require.EqualValues(t, 3, subnet0.LocalSubnets[2].DaemonID)
+}
