@@ -27,6 +27,7 @@ type SystemUser struct {
 	Groups []*SystemGroup `pg:"many2many:system_user_to_group,fk:user_id,join_fk:group_id"`
 }
 
+// Represents an association table between the system user and group tables.
 type SystemUserToGroup struct {
 	UserID  int `pg:",pk,notnull,on_delete:CASCADE"`
 	GroupID int `pg:",pk,notnull,on_delete:CASCADE"`
@@ -150,6 +151,20 @@ func UpdateUser(db *pg.DB, user *SystemUser) (conflict bool, err error) {
 	return conflict, err
 }
 
+// Deletes existing user from the database. The returned error value indicates if
+// the deleted user information does not match any existing user in the
+// database.
+func DeleteUser(db *pg.DB, user *SystemUser) (err error) {
+	result, err := db.Model(user).WherePK().Delete()
+	if err != nil {
+		err = pkgerrors.Wrapf(err, "database operation error while trying to delete user %s", user.Identity())
+	} else if result.RowsAffected() <= 0 {
+		err = pkgerrors.Wrapf(ErrNotExists, "user with ID %s does not exist", user.Identity())
+	}
+
+	return err
+}
+
 // Sets new password for the given user id.
 func SetPassword(db *pg.DB, id int, password string) (err error) {
 	user := SystemUser{
