@@ -5,6 +5,7 @@ import (
 	"testing"
 
 	faker "github.com/brianvoe/gofakeit"
+	pkgerrors "github.com/pkg/errors"
 	"github.com/stretchr/testify/require"
 	dbops "isc.org/stork/server/database"
 	dbtest "isc.org/stork/server/database/test"
@@ -154,6 +155,31 @@ func TestCreateConflict(t *testing.T) {
 	require.Error(t, err)
 }
 
+// Test that the user is successfully deleted.
+func TestDeleteUser(t *testing.T) {
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	// Create new user.
+	user := &SystemUser{
+		Email:    "jan@example.org",
+		Lastname: "Kowalski",
+		Name:     "Jan",
+		Password: "pass",
+	}
+	con, err := CreateUser(db, user)
+	require.False(t, con)
+	require.NoError(t, err)
+
+	err = DeleteUser(db, user)
+	require.NoError(t, err)
+
+	// An attempt to delete the same user should result in an error.
+	err = DeleteUser(db, user)
+	require.Error(t, err)
+	require.ErrorIs(t, pkgerrors.Cause(err), ErrNotExists)
+}
+
 // Tests that password can be modified.
 func TestSetPassword(t *testing.T) {
 	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
@@ -379,10 +405,10 @@ func TestUserGroups(t *testing.T) {
 		Password: "pass",
 		Groups: []*SystemGroup{
 			{
-				ID: 1,
+				ID: SuperAdminGroupID,
 			},
 			{
-				ID: 2,
+				ID: AdminGroupID,
 			},
 		},
 	}
@@ -472,3 +498,33 @@ func TestAddToGroupByID(t *testing.T) {
 	require.NoError(t, err)
 	require.False(t, added)
 }
+
+// Test that the user is successfully deleted if added to super-admin group.
+func TestDeleteUserInGroup(t *testing.T) {
+	db, _, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	// Create new user.
+	user := &SystemUser{
+		Email:    "jan@example.org",
+		Lastname: "Kowalski",
+		Name:     "Jan",
+		Password: "pass",
+		Groups: []*SystemGroup{
+			{
+				ID: SuperAdminGroupID,
+			},
+		},
+	}
+	con, err := CreateUser(db, user)
+	require.False(t, con)
+	require.NoError(t, err)
+
+	err = DeleteUser(db, user)
+	require.NoError(t, err)
+
+	// An attempt to delete the same user should result in an error.
+	err = DeleteUser(db, user)
+	require.Error(t, err)
+	require.ErrorIs(t, pkgerrors.Cause(err), ErrNotExists)
+}
