@@ -29,7 +29,7 @@ func TestMiddlewareNewSession(t *testing.T) {
 	_, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
 	defer teardown()
 
-	mgr, err := NewSessionMgr(&dbSettings.BaseDatabaseSettings)
+	mgr, err := NewSessionMgr(dbSettings)
 	require.NoError(t, err)
 
 	req := httptest.NewRequest("GET", "http://example.com/foo", nil)
@@ -81,6 +81,7 @@ func TestMiddlewareNewSession(t *testing.T) {
 	middlewareFunc := mgr.SessionMiddleware(http.HandlerFunc(handler))
 	middlewareFunc.ServeHTTP(w, req)
 	resp := w.Result()
+	defer resp.Body.Close()
 	require.Equal(t, resp.StatusCode, 200)
 
 	// Check that the session cookie was set in the response.
@@ -96,7 +97,7 @@ func TestLoad(t *testing.T) {
 	_, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
 	defer teardown()
 
-	mgr, err := NewSessionMgr(&dbSettings.BaseDatabaseSettings)
+	mgr, err := NewSessionMgr(dbSettings)
 	require.NoError(t, err)
 
 	ctx := context.Background()
@@ -111,7 +112,7 @@ func TestLogOutUser(t *testing.T) {
 	defer teardown()
 
 	// Create session manager.
-	mgr, err := NewSessionMgr(&dbSettings.BaseDatabaseSettings)
+	mgr, err := NewSessionMgr(dbSettings)
 	require.NoError(t, err)
 
 	ctx := context.Background()
