@@ -232,11 +232,11 @@ func TestGetMachineAndAppsState(t *testing.T) {
 	fa.MachineState = &agentcomm.State{
 		Apps: []*agentcomm.App{
 			{
-				Type:         dbmodel.AppTypeKea,
+				Type:         dbmodel.AppTypeKea.String(),
 				AccessPoints: agentcomm.MakeAccessPoint(dbmodel.AccessPointControl, "1.2.3.4", "", 123),
 			},
 			{
-				Type:         dbmodel.AppTypeBind9,
+				Type:         dbmodel.AppTypeBind9.String(),
 				AccessPoints: agentcomm.MakeAccessPoint(dbmodel.AccessPointControl, "1.2.3.4", "abcd", 124),
 			},
 		},
@@ -250,8 +250,8 @@ func TestGetMachineAndAppsState(t *testing.T) {
 	require.IsType(t, &services.GetMachineStateOK{}, rsp)
 	okRsp := rsp.(*services.GetMachineStateOK)
 	require.Len(t, okRsp.Payload.Apps, 2)
-	require.Equal(t, dbmodel.AppTypeKea, okRsp.Payload.Apps[0].Type)
-	require.Equal(t, dbmodel.AppTypeBind9, okRsp.Payload.Apps[1].Type)
+	require.Equal(t, dbmodel.AppTypeKea.String(), okRsp.Payload.Apps[0].Type)
+	require.Equal(t, dbmodel.AppTypeBind9.String(), okRsp.Payload.Apps[1].Type)
 }
 
 func TestCreateMachine(t *testing.T) {
@@ -687,6 +687,7 @@ func TestGetMachine(t *testing.T) {
 	require.Equal(t, m2.ID, okRsp.Payload.ID)
 	require.Len(t, okRsp.Payload.Apps, 1)
 	require.Equal(t, s.ID, okRsp.Payload.Apps[0].ID)
+	require.Len(t, okRsp.Payload.Apps[0].AccessPoints, 1)
 }
 
 func TestUpdateMachine(t *testing.T) {
@@ -1097,7 +1098,7 @@ func TestRestGetApps(t *testing.T) {
 	// to see where those counters are set.
 	require.Len(t, okRsp.Payload.Items, 2)
 	for _, app := range okRsp.Payload.Items {
-		if app.Type == dbmodel.AppTypeKea {
+		if app.Type == dbmodel.AppTypeKea.String() {
 			require.Equal(t, "fancy-app", app.Name)
 			appKea := app.Details.AppKea
 			require.Len(t, appKea.Daemons, 1)
@@ -1109,7 +1110,7 @@ func TestRestGetApps(t *testing.T) {
 			require.Equal(t, "kea-dhcp4", daemon.LogTargets[0].Name)
 			require.Equal(t, "debug", daemon.LogTargets[0].Severity)
 			require.Equal(t, "/tmp/log", daemon.LogTargets[0].Output)
-		} else if app.Type == dbmodel.AppTypeBind9 {
+		} else if app.Type == dbmodel.AppTypeBind9.String() {
 			require.Equal(t, "another-fancy-app", app.Name)
 			appBind9 := app.Details.AppBind9
 			daemon := appBind9.Daemon
@@ -1972,7 +1973,7 @@ func TestGetKeaStorages(t *testing.T) {
             ]
         }
     }`
-	keaConfig, err := keaconfig.NewFromJSON(configString)
+	keaConfig, err := keaconfig.NewConfig(configString)
 	require.NoError(t, err)
 	require.NotNil(t, keaConfig)
 
@@ -2081,7 +2082,7 @@ func TestGetKeaStoragesLeaseDatabase(t *testing.T) {
             }
         }
     }`
-	keaConfig, err := keaconfig.NewFromJSON(configString)
+	keaConfig, err := keaconfig.NewConfig(configString)
 	require.NoError(t, err)
 	require.NotNil(t, keaConfig)
 
@@ -2110,7 +2111,7 @@ func TestGetKeaStoragesForensicDatabase(t *testing.T) {
             ]
         }
     }`
-	keaConfig, err := keaconfig.NewFromJSON(configString)
+	keaConfig, err := keaconfig.NewConfig(configString)
 	require.NoError(t, err)
 	require.NotNil(t, keaConfig)
 
@@ -2272,3 +2273,129 @@ func TestGetMachineDumpReturnsExpectedFilename(t *testing.T) {
 	require.EqualValues(t, extension, ".tar.gz")
 	require.LessOrEqual(t, time.Now().UTC().Sub(timestamp).Seconds(), float64(10))
 }
+
+// Test that only super-administrators can fetch the authentication key of the
+// access point.
+func TestGetAccessPointKeyIsRestrictedToSuperAdmins(t *testing.T) {
+	// Arrange
+	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	settings := &RestAPISettings{}
+	rapi, _ := NewRestAPI(settings, dbSettings, db)
+
+	// Create an admin user.
+	ctx, _ := rapi.SessionManager.Load(context.Background(), "")
+	user := &dbmodel.SystemUser{
+		Login:    "foo",
+		Password: "bar",
+		Name:     "baz",
+		Lastname: "boz",
+		Groups:   []*dbmodel.SystemGroup{{ID: dbmodel.AdminGroupID}},
+	}
+	_, _ = dbmodel.CreateUser(rapi.DB, user)
+	_ = rapi.SessionManager.LoginHandler(ctx, user)
+
+	// Act
+	rsp := rapi.GetAccessPointKey(ctx, services.GetAccessPointKeyParams{
+		AppID: 42,
+		Type:  dbmodel.AccessPointControl,
+	})
+
+	// Assert
+	defaultRsp, ok := rsp.(*services.GetAccessPointKeyDefault)
+	require.True(t, ok)
+	require.Equal(t, http.StatusForbidden, getStatusCode(*defaultRsp))
+}
+
+// Test that the HTTP 500 Internal Server Error status is returned if the
+// database is not available.
+func TestGetAccessPointKeyForInvalidDatabase(t *testing.T) {
+	// Arrange
+	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
+
+	settings := &RestAPISettings{}
+	rapi, _ := NewRestAPI(settings, dbSettings, db)
+
+	ctx, _ := rapi.SessionManager.Load(context.Background(), "")
+	user, _ := dbmodel.GetUserByID(rapi.DB, 1)
+	_ = rapi.SessionManager.LoginHandler(ctx, user)
+
+	// Act
+	teardown()
+	rsp := rapi.GetAccessPointKey(ctx, services.GetAccessPointKeyParams{
+		AppID: 42,
+		Type:  dbmodel.AccessPointControl,
+	})
+
+	// Assert
+	defaultRsp, ok := rsp.(*services.GetAccessPointKeyDefault)
+	require.True(t, ok)
+	require.Equal(t, http.StatusInternalServerError, getStatusCode(*defaultRsp))
+}
+
+// Test that the HTTP 404 Not Found status is returned if the access point
+// doesn't exist.
+func TestGetAccessPointKeyForMissingEntry(t *testing.T) {
+	// Arrange
+	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	settings := &RestAPISettings{}
+	rapi, _ := NewRestAPI(settings, dbSettings, db)
+
+	ctx, _ := rapi.SessionManager.Load(context.Background(), "")
+	user, _ := dbmodel.GetUserByID(rapi.DB, 1)
+	_ = rapi.SessionManager.LoginHandler(ctx, user)
+
+	// Act
+	rsp := rapi.GetAccessPointKey(ctx, services.GetAccessPointKeyParams{
+		AppID: 42,
+		Type:  dbmodel.AccessPointControl,
+	})
+
+	// Assert
+	defaultRsp, ok := rsp.(*services.GetAccessPointKeyDefault)
+	require.True(t, ok)
+	require.Equal(t, http.StatusNotFound, getStatusCode(*defaultRsp))
+}
+
+// Test that the authentication key of a given access point is fetched properly.
+func TestGetAccessPointKey(t *testing.T) {
+	// Arrange
+	db, dbSettings, teardown := dbtest.SetupDatabaseTestCase(t)
+	defer teardown()
+
+	settings := &RestAPISettings{}
+	rapi, _ := NewRestAPI(settings, dbSettings, db)
+
+	ctx, _ := rapi.SessionManager.Load(context.Background(), "")
+	user, _ := dbmodel.GetUserByID(rapi.DB, 1)
+	_ = rapi.SessionManager.LoginHandler(ctx, user)
+
+	machine := &dbmodel.Machine{Address: "localhost", AgentPort: 8080}
+	_ = dbmodel.AddMachine(db, machine)
+	app := &dbmodel.App{
+		MachineID: machine.ID,
+		Type:      dbmodel.AppTypeBind9,
+		AccessPoints: []*dbmodel.AccessPoint{{
+			Type:              dbmodel.AccessPointControl,
+			Address:           "127.0.0.1",
+			Port:              8080,
+			Key:               "secret",
+			UseSecureProtocol: true,
+		}},
+	}
+	_, _ = dbmodel.AddApp(db, app)
+
+	// Act
+	rsp := rapi.GetAccessPointKey(ctx, services.GetAccessPointKeyParams{
+		AppID: app.ID,
+		Type:  dbmodel.AccessPointControl,
+	})
+
+	// Assert
+	okRsp, ok := rsp.(*services.GetAccessPointKeyOK)
+	require.True(t, ok)
+	require.EqualValues(t, "secret", okRsp.Payload)
+}
