@@ -200,6 +200,16 @@ func TestSearchRecords(t *testing.T) {
 			Subnets: []dbmodel.Subnet{
 				{
 					Prefix: "5001:db8:1::/64",
+					LocalSubnets: []*dbmodel.LocalSubnet{
+						{
+							DaemonID: a46.Daemons[1].ID,
+						},
+					},
+				},
+			},
+			LocalSharedNetworks: []*dbmodel.LocalSharedNetwork{
+				{
+					DaemonID: a46.Daemons[1].ID,
 				},
 			},
 		},
@@ -214,6 +224,11 @@ func TestSearchRecords(t *testing.T) {
 					UpperBound: "192.118.0.200",
 				},
 			},
+			LocalSubnets: []*dbmodel.LocalSubnet{
+				{
+					DaemonID: a46.Daemons[0].ID,
+				},
+			},
 		},
 		{
 			Prefix: "3001:db8:1::/64",
@@ -223,6 +238,11 @@ func TestSearchRecords(t *testing.T) {
 					UpperBound: "3001:db8:1:0:ffff::ffff",
 				},
 			},
+			LocalSubnets: []*dbmodel.LocalSubnet{
+				{
+					DaemonID: a46.Daemons[1].ID,
+				},
+			},
 		},
 	}
 	_, err = dbmodel.CommitNetworksIntoDB(db, []dbmodel.SharedNetwork{}, []dbmodel.Subnet{appSubnets[0]}, a46.Daemons[0])
