@@ -89,6 +89,12 @@ func TestGetSubnets(t *testing.T) {
 					UpperBound: "192.168.0.200",
 				},
 			},
+			LocalSubnets: []*dbmodel.LocalSubnet{
+				{
+					DaemonID:      a4.Daemons[0].ID,
+					LocalSubnetID: 1,
+				},
+			},
 		},
 	}
 
@@ -128,6 +134,12 @@ func TestGetSubnets(t *testing.T) {
 	appSubnets = []dbmodel.Subnet{
 		{
 			Prefix: "2001:db8:1::/64",
+			LocalSubnets: []*dbmodel.LocalSubnet{
+				{
+					DaemonID:      a6.Daemons[0].ID,
+					LocalSubnetID: 2,
+				},
+			},
 		},
 	}
 	_, err = dbmodel.CommitNetworksIntoDB(db, []dbmodel.SharedNetwork{}, appSubnets, a6.Daemons[0])
@@ -193,6 +205,17 @@ func TestGetSubnets(t *testing.T) {
 		Subnets: []dbmodel.Subnet{
 			{
 				Prefix: "5001:db8:1::/64",
+				LocalSubnets: []*dbmodel.LocalSubnet{
+					{
+						DaemonID:      a46.Daemons[1].ID,
+						LocalSubnetID: 21,
+					},
+				},
+			},
+		},
+		LocalSharedNetworks: []*dbmodel.LocalSharedNetwork{
+			{
+				DaemonID: a46.Daemons[1].ID,
 			},
 		},
 	}
@@ -209,6 +232,12 @@ func TestGetSubnets(t *testing.T) {
 				UpperBound: "192.118.0.200",
 			},
 		},
+		LocalSubnets: []*dbmodel.LocalSubnet{
+			{
+				DaemonID:      a46.Daemons[0].ID,
+				LocalSubnetID: 3,
+			},
+		},
 		AddrUtilization: 420,
 	}
 
@@ -224,6 +253,12 @@ func TestGetSubnets(t *testing.T) {
 				UpperBound: "3001:db8:1:0:ffff::ffff",
 			},
 		},
+		LocalSubnets: []*dbmodel.LocalSubnet{
+			{
+				DaemonID:      a46.Daemons[1].ID,
+				LocalSubnetID: 4,
+			},
+		},
 		AddrUtilization: 240,
 		PdUtilization:   420,
 	}
@@ -413,10 +448,10 @@ func TestGetSharedNetworks(t *testing.T) {
 								"name": "mouse",
 								"subnet4": []map[string]interface{}{{
 									"id":     12,
-									"subnet": "192.2.0.0/24",
+									"subnet": "192.3.0.0/24",
 								}, {
 									"id":     13,
-									"subnet": "192.3.0.0/24",
+									"subnet": "192.2.0.0/24",
 								}},
 							}},
 						},
@@ -435,6 +470,17 @@ func TestGetSharedNetworks(t *testing.T) {
 			Subnets: []dbmodel.Subnet{
 				{
 					Prefix: "192.1.0.0/24",
+					LocalSubnets: []*dbmodel.LocalSubnet{
+						{
+							DaemonID:      a4.Daemons[0].ID,
+							LocalSubnetID: 11,
+						},
+					},
+				},
+			},
+			LocalSharedNetworks: []*dbmodel.LocalSharedNetwork{
+				{
+					DaemonID: a4.Daemons[0].ID,
 				},
 			},
 		},
@@ -443,10 +489,22 @@ func TestGetSharedNetworks(t *testing.T) {
 			Family: 4,
 			Subnets: []dbmodel.Subnet{
 				{
-					Prefix: "192.2.0.0/24",
+					Prefix: "192.3.0.0/24",
+					LocalSubnets: []*dbmodel.LocalSubnet{
+						{
+							DaemonID:      a4.Daemons[0].ID,
+							LocalSubnetID: 12,
+						},
+					},
 				},
 				{
-					Prefix: "192.3.0.0/24",
+					Prefix: "192.2.0.0/24",
+					LocalSubnets: []*dbmodel.LocalSubnet{
+						{
+							DaemonID:      a4.Daemons[0].ID,
+							LocalSubnetID: 13,
+						},
+					},
 				},
 			},
 		},
@@ -475,10 +533,10 @@ func TestGetSharedNetworks(t *testing.T) {
 								"name": "fox",
 								"subnet6": []map[string]interface{}{{
 									"id":     21,
-									"subnet": "5001:db8:1::/64",
+									"subnet": "6001:db8:1::/64",
 								}, {
 									"id":     22,
-									"subnet": "6001:db8:1::/64",
+									"subnet": "5001:db8:1::/64",
 								}},
 							}},
 						},
@@ -496,10 +554,22 @@ func TestGetSharedNetworks(t *testing.T) {
 			Family: 6,
 			Subnets: []dbmodel.Subnet{
 				{
-					Prefix: "5001:db8:1::/64",
+					Prefix: "6001:db8:1::/64",
+					LocalSubnets: []*dbmodel.LocalSubnet{
+						{
+							DaemonID:      a6.Daemons[0].ID,
+							LocalSubnetID: 21,
+						},
+					},
 				},
 				{
-					Prefix: "6001:db8:1::/64",
+					Prefix: "5001:db8:1::/64",
+					LocalSubnets: []*dbmodel.LocalSubnet{
+						{
+							DaemonID:      a6.Daemons[0].ID,
+							LocalSubnetID: 22,
+						},
+					},
 				},
 			},
 		},
@@ -519,10 +589,22 @@ func TestGetSharedNetworks(t *testing.T) {
 		switch net.Name {
 		case "frog":
 			require.Len(t, net.Subnets, 1)
+			require.EqualValues(t, 1, net.Subnets[0].ID)
+			require.Equal(t, "192.1.0.0/24", net.Subnets[0].Subnet)
 		case "mouse":
 			require.Len(t, net.Subnets, 2)
+			// subnets should be sorted by prefix, not by ID
+			require.EqualValues(t, 3, net.Subnets[0].ID)
+			require.Equal(t, "192.2.0.0/24", net.Subnets[0].Subnet)
+			require.EqualValues(t, 2, net.Subnets[1].ID)
+			require.Equal(t, "192.3.0.0/24", net.Subnets[1].Subnet)
 		case "fox":
 			require.Len(t, net.Subnets, 2)
+			// subnets should be sorted by prefix, not by ID
+			require.EqualValues(t, 5, net.Subnets[0].ID)
+			require.Equal(t, "5001:db8:1::/64", net.Subnets[0].Subnet)
+			require.EqualValues(t, 4, net.Subnets[1].ID)
+			require.Equal(t, "6001:db8:1::/64", net.Subnets[1].Subnet)
 		}
 	}
 
