@@ -10,6 +10,8 @@ import (
 	dbtest "isc.org/stork/server/database/test"
 	"isc.org/stork/server/gen/models"
 	"isc.org/stork/server/gen/restapi/operations/users"
+	"isc.org/stork/server/hookmanager"
+	storkutil "isc.org/stork/util"
 )
 
 // Tests that create user account without necessary fields is rejected via REST API.
@@ -30,7 +32,7 @@ func TestCreateUserConflict(t *testing.T) {
 	params := users.CreateUserParams{
 		Account: &models.UserAccount{
 			User:     newRestUser(su),
-			Password: models.Password("pass"),
+			Password: storkutil.Ptr(models.Password("pass")),
 		},
 	}
 
@@ -117,7 +119,7 @@ func TestCreateUser(t *testing.T) {
 	params := users.CreateUserParams{
 		Account: &models.UserAccount{
 			User:     newRestUser(su),
-			Password: models.Password("pass"),
+			Password: storkutil.Ptr(models.Password("pass")),
 		},
 	}
 
@@ -535,13 +537,13 @@ func TestUpdateUserInvalidUserID(t *testing.T) {
 	require.False(t, con)
 	require.NoError(t, err)
 
-	// An attempt to update non-existing user (non macthing ID) should
+	// An attempt to update non-existing user (non matching ID) should
 	// result in an error 409.
 	su.ID = 123
 	params := users.UpdateUserParams{
 		Account: &models.UserAccount{
 			User:     newRestUser(su),
-			Password: models.Password("pass"),
+			Password: storkutil.Ptr(models.Password("pass")),
 		},
 	}
 	rsp := rapi.UpdateUser(ctx, params)
@@ -576,7 +578,7 @@ func TestUpdateUser(t *testing.T) {
 	params := users.UpdateUserParams{
 		Account: &models.UserAccount{
 			User:     newRestUser(su),
-			Password: models.Password("pass"),
+			Password: storkutil.Ptr(models.Password("pass")),
 		},
 	}
 	rsp := rapi.UpdateUser(ctx, params)
@@ -586,6 +588,20 @@ func TestUpdateUser(t *testing.T) {
 	returned, err := dbmodel.GetUserByID(db, su.ID)
 	require.NoError(t, err)
 	require.Equal(t, su.Lastname, returned.Lastname)
+
+	// An attempt to update non-existing user (non matching ID) should
+	// result in an error 409.
+	su.ID = 123
+	params = users.UpdateUserParams{
+		Account: &models.UserAccount{
+			User:     newRestUser(su),
+			Password: storkutil.Ptr(models.Password("pass")),
+		},
+	}
+	rsp = rapi.UpdateUser(ctx, params)
+	require.IsType(t, &users.UpdateUserDefault{}, rsp)
+	defaultRsp := rsp.(*users.UpdateUserDefault)
+	require.Equal(t, 409, getStatusCode(*defaultRsp))
 }
 
 // Tests that update user password with missing data is rejected via REST API.
@@ -643,8 +659,8 @@ func TestUpdateUserPassword(t *testing.T) {
 	params := users.UpdateUserPasswordParams{
 		ID: int64(user.ID),
 		Passwords: &models.PasswordChange{
-			Newpassword: models.Password("updated"),
-			Oldpassword: models.Password("pass"),
+			Newpassword: storkutil.Ptr(models.Password("updated")),
+			Oldpassword: storkutil.Ptr(models.Password("pass")),
 		},
 	}
 	rsp := rapi.UpdateUserPassword(ctx, params)
@@ -656,8 +672,8 @@ func TestUpdateUserPassword(t *testing.T) {
 	params = users.UpdateUserPasswordParams{
 		ID: int64(user.ID),
 		Passwords: &models.PasswordChange{
-			Newpassword: models.Password("updated"),
-			Oldpassword: models.Password("pass"),
+			Newpassword: storkutil.Ptr(models.Password("updated")),
+			Oldpassword: storkutil.Ptr(models.Password("pass")),
 		},
 	}
 	rsp = rapi.UpdateUserPassword(ctx, params)
@@ -797,7 +813,8 @@ func TestCreateSessionEmptyParams(t *testing.T) {
 	defer teardown()
 
 	ctx := context.Background()
-	rapi, _ := NewRestAPI(dbSettings, db)
+	hookManager := hookmanager.NewHookManager()
+	rapi, _ := NewRestAPI(dbSettings, db, hookManager)
 
 	user := &dbmodel.SystemUser{
 		Email:    "jan@example.org",
@@ -821,7 +838,8 @@ func TestCreateSessionInvalidCredentials(t *testing.T) {
 	defer teardown()
 
 	ctx := context.Background()
-	rapi, _ := NewRestAPI(dbSettings, db)
+	hookManager := hookmanager.NewHookManager()
+	rapi, _ := NewRestAPI(dbSettings, db, hookManager)
 
 	user := &dbmodel.SystemUser{
 		Email:    "jan@example.org",
@@ -852,7 +870,8 @@ func TestCreateSession(t *testing.T) {
 	defer teardown()
 
 	ctx := context.Background()
-	rapi, _ := NewRestAPI(dbSettings, db)
+	hookManager := hookmanager.NewHookManager()
+	rapi, _ := NewRestAPI(dbSettings, db, hookManager)
 
 	user := &dbmodel.SystemUser{
 		Email:    "jan@example.org",
