@@ -4,6 +4,7 @@ import (
 	"bytes"
 	"encoding/hex"
 	"fmt"
+	"io/fs"
 	"os"
 	"os/exec"
 	"path"
@@ -183,6 +184,7 @@ func SetupLogging() {
 // improve testability and allow mock the operating system operations.
 type CommandExecutor interface {
 	Output(string, ...string) ([]byte, error)
+	LookPath(string) (string, error)
 }
 
 // Executes the given command in the operating system.
@@ -199,6 +201,11 @@ func (e *systemCommandExecutor) Output(command string, args ...string) ([]byte,
 	return exec.Command(command, args...).Output()
 }
 
+// Looks for a given command in the system PATH and returns absolute path if found.
+func (e *systemCommandExecutor) LookPath(command string) (string, error) {
+	return exec.LookPath(command)
+}
+
 // Convert bytes to hex string.
 func BytesToHex(bytesArray []byte) string {
 	var buf bytes.Buffer
@@ -216,18 +223,24 @@ func HexToBytes(hexString string) []byte {
 }
 
 // Prompts for secret in a terminal. The typed characters are not printed to
-// standard output. If a terminal is unavailable (no TTY attached), it causes
-// panic.
-func GetSecretInTerminal(prompt string) string {
+// standard output. If a terminal is unavailable (no TTY attached), returns
+// an error.
+func GetSecretInTerminal(prompt string) (string, error) {
 	// Prompt the user for a secret
+	descriptor := int(os.Stdin.Fd())
+	if !term.IsTerminal(descriptor) {
+		return "", errors.Errorf("Not running in a terminal")
+	}
+
 	fmt.Print(prompt)
-	pass, err := term.ReadPassword(0)
+	pass, err := term.ReadPassword(descriptor)
 	fmt.Print("\n")
 
 	if err != nil {
-		log.Fatal(err.Error())
+		err = errors.Wrapf(err, "cannot read password for the '%d' descriptor", descriptor)
+		return "", err
 	}
-	return string(pass)
+	return string(pass), nil
 }
 
 // Read a file and resolve all include statements.
@@ -372,6 +385,12 @@ func IsNilPtr(obj interface{}) bool {
 	return obj == nil || reflect.ValueOf(obj).Kind() == reflect.Ptr && reflect.ValueOf(obj).IsNil()
 }
 
+// Returns a pointer to the specified value. It is useful to
+// create pointers from the literals.
+func Ptr[T any](value T) *T {
+	return &value
+}
+
 // Checks if the specified value is a whole number.
 func IsWholeNumber(value interface{}) bool {
 	if value == nil {
@@ -385,3 +404,34 @@ func IsWholeNumber(value interface{}) bool {
 		return false
 	}
 }
+
+// Combine multiple errors into a single one.
+func CombineErrors(topErrorMsg string, errs []error) error {
+	var noNils []error
+	for _, err := range errs {
+		if err != nil {
+			noNils = append(noNils, err)
+		}
+	}
+
+	if len(noNils) == 0 {
+		return nil
+	}
+
+	combinedErr := errors.New(topErrorMsg)
+
+	for _, err := range noNils {
+		combinedErr = errors.WithMessage(combinedErr, err.Error())
+	}
+
+	return combinedErr
+}
+
+// Checks if the path is a socket.
+func IsSocket(path string) bool {
+	s, err := os.Stat(path)
+	if err != nil {
+		return false
+	}
+	return s.Mode().Type() == fs.ModeSocket
+}
