@@ -241,14 +241,14 @@ def test_start_uses_service_names():
     up_mock.assert_called_once_with(*services)
 
 
-def test_stop_calls_proper_command_and_removes_volumes():
+def test_down_calls_proper_command_and_removes_volumes():
     # Arrange
     compose = DockerCompose("project-dir")
     mock = MagicMock()
     compose._call_command = mock
     base_cmd = compose.docker_compose_command()
     # Act
-    compose.stop()
+    compose.down()
     # Assert
     mock.assert_called_once()
     cmd = mock.call_args.kwargs["cmd"]
@@ -746,6 +746,7 @@ def test_get_pid():
     assert cmd[-2] == "pid"
     assert cmd[-1] == "kea-dhcp4"
 
+
 def test_get_pid_unparsable_pid():
     # Arrange
     compose = DockerCompose("project-dir")
@@ -757,9 +758,10 @@ def test_get_pid_unparsable_pid():
     # Assert
     assert pid is None
 
-def test_is_premium_reads_config():
+
+def test_is_enabled_reads_config():
     # Arrange
-    compose = DockerCompose("project-dir")
+    compose = DockerCompose("project-dir", profiles=["premium"])
     mock = MagicMock()
     config = {
         'services': {
@@ -773,7 +775,8 @@ def test_is_premium_reads_config():
                 'profiles': [
                     'non-premium'
                 ]
-            }
+            },
+            'baz': {}
         }
     }
     config_yaml = yaml.safe_dump(config)
@@ -781,19 +784,22 @@ def test_is_premium_reads_config():
     compose._call_command = mock
     base_cmd = compose.docker_compose_command()
     # Act
-    is_foo_premium = compose.is_premium("foo")
-    is_bar_premium = compose.is_premium("bar")
+    is_foo_enabled = compose.is_enabled("foo")
+    is_bar_enabled = compose.is_enabled("bar")
+    is_baz_enabled = compose.is_enabled("baz")
     # Assert
     # Checks if config is memoized.
     mock.assert_called_once()
     cmd = mock.call_args.kwargs["cmd"]
     assert " ".join(cmd).startswith(" ".join(base_cmd))
     assert cmd[-1] == "config"
-    assert is_foo_premium
-    assert not is_bar_premium
+    assert is_foo_enabled
+    assert not is_bar_enabled
+    assert is_baz_enabled
 
 
-def test_is_premium_raises_for_unknown_service():
+def test_is_enabled_for_unknown_service():
+    # Arrange
     compose = DockerCompose("project-dir")
     mock = MagicMock()
     config = {
@@ -803,8 +809,7 @@ def test_is_premium_raises_for_unknown_service():
     mock.return_value = (0, config_yaml, "")
     compose._call_command = mock
     # Act & Assert
-    with pytest.raises(LookupError):
-        compose.is_premium("foo")
+    assert not compose.is_enabled("foo")
 
 
 @patch("subprocess.run")
@@ -813,7 +818,8 @@ def test_call_command_passes_command(patch: MagicMock):
     compose._call_command(["foo", "bar"])
     patch.assert_called_once()
     cmd = patch.call_args.args[0]
-    assert tuple(cmd) == ("foo", "bar")
+    assert tuple(cmd[-2:]) == ("foo", "bar")
+
 
 @patch("subprocess.run")
 def test_call_command_adds_env_vars(patch: MagicMock):
@@ -841,7 +847,7 @@ def test_call_command_captures_output_by_default(patch: MagicMock):
 
 
 @patch("subprocess.run", return_value=subprocess_result_mock(0, b"foo\n", b"bar\n"))
-def test_call_command_suppreses_capturing_output(patch: MagicMock):
+def test_call_command_suppresses_capturing_output(patch: MagicMock):
     compose = DockerCompose("project-dir")
     status, stdout, stderr = compose._call_command([], capture_output=False)
     patch.assert_called_once()
@@ -862,7 +868,7 @@ def test_call_command_checks_output_by_default(patch: MagicMock):
 
 
 @patch("subprocess.run")
-def test_call_command_suppresses_checking_ouput(patch: MagicMock):
+def test_call_command_suppresses_checking_output(patch: MagicMock):
     compose = DockerCompose("project-dir")
     compose._call_command([], check=False)
     patch.assert_called_once()
