@@ -29,11 +29,13 @@ import { DropdownModule } from 'primeng/dropdown'
 import { MultiSelectModule } from 'primeng/multiselect'
 import { ConfirmDialogModule } from 'primeng/confirmdialog'
 import { HostFormComponent } from '../host-form/host-form.component'
-import { Hosts } from '../backend/model/hosts'
 import { DhcpOptionFormComponent } from '../dhcp-option-form/dhcp-option-form.component'
 import { DhcpOptionSetFormComponent } from '../dhcp-option-set-form/dhcp-option-set-form.component'
 import { DhcpOptionSetViewComponent } from '../dhcp-option-set-view/dhcp-option-set-view.component'
 import { TreeModule } from 'primeng/tree'
+import { DhcpClientClassSetFormComponent } from '../dhcp-client-class-set-form/dhcp-client-class-set-form.component'
+import { DhcpClientClassSetViewComponent } from '../dhcp-client-class-set-view/dhcp-client-class-set-view.component'
+import { ChipsModule } from 'primeng/chips'
 
 class MockParamMap {
     get(name: string): string | null {
@@ -56,6 +58,8 @@ describe('HostsPageComponent', () => {
         TestBed.configureTestingModule({
             providers: [DHCPService, UntypedFormBuilder, ConfirmationService, MessageService],
             imports: [
+                ButtonModule,
+                ChipsModule,
                 FormsModule,
                 TableModule,
                 HttpClientTestingModule,
@@ -78,6 +82,7 @@ describe('HostsPageComponent', () => {
                 FormsModule,
                 FieldsetModule,
                 ProgressSpinnerModule,
+                TableModule,
                 ToggleButtonModule,
                 ButtonModule,
                 CheckboxModule,
@@ -96,6 +101,8 @@ describe('HostsPageComponent', () => {
                 HostTabComponent,
                 IdentifierComponent,
                 HostFormComponent,
+                DhcpClientClassSetFormComponent,
+                DhcpClientClassSetViewComponent,
                 DhcpOptionFormComponent,
                 DhcpOptionSetFormComponent,
                 DhcpOptionSetViewComponent,
@@ -650,6 +657,7 @@ describe('HostsPageComponent', () => {
                     },
                 },
             ],
+            clientClasses: ['router', 'cable-modem'],
         }
         const okResp: any = {
             status: 200,
@@ -701,6 +709,7 @@ describe('HostsPageComponent', () => {
                     },
                 },
             ],
+            clientClasses: ['router', 'cable-modem'],
         }
         const okResp: any = {
             status: 200,
@@ -752,6 +761,7 @@ describe('HostsPageComponent', () => {
                     },
                 },
             ],
+            clientClasses: ['router', 'cable-modem'],
         }
         const okResp: any = {
             status: 200,
@@ -843,6 +853,7 @@ describe('HostsPageComponent', () => {
                     },
                 },
             ],
+            clientClasses: ['router', 'cable-modem'],
             host: component.hosts[0],
         }
         const okResp: any = {
@@ -934,6 +945,7 @@ describe('HostsPageComponent', () => {
                     },
                 },
             ],
+            clientClasses: ['router', 'cable-modem'],
             host: component.hosts[0],
         }
         const okResp: any = {
@@ -1073,4 +1085,14 @@ describe('HostsPageComponent', () => {
         fixture.detectChanges()
         expect(dhcpApi.getHosts).toHaveBeenCalled()
     })
+
+    it('should have breadcrumbs', () => {
+        const breadcrumbsElement = fixture.debugElement.query(By.directive(BreadcrumbsComponent))
+        expect(breadcrumbsElement).not.toBeNull()
+        const breadcrumbsComponent = breadcrumbsElement.componentInstance as BreadcrumbsComponent
+        expect(breadcrumbsComponent).not.toBeNull()
+        expect(breadcrumbsComponent.items).toHaveSize(2)
+        expect(breadcrumbsComponent.items[0].label).toEqual('DHCP')
+        expect(breadcrumbsComponent.items[1].label).toEqual('Host Reservations')
+    })
 })
