@@ -33,6 +33,7 @@ import { AppOverviewComponent } from '../app-overview/app-overview.component'
 import { ButtonModule } from 'primeng/button'
 import { DataViewModule } from 'primeng/dataview'
 import { ToggleButtonModule } from 'primeng/togglebutton'
+import { ProgressSpinnerModule } from 'primeng/progressspinner'
 
 class Details {
     daemons: any = [
@@ -124,6 +125,7 @@ describe('KeaAppTabComponent', () => {
                 FormsModule,
                 ToggleButtonModule,
                 DataViewModule,
+                ProgressSpinnerModule,
             ],
             declarations: [
                 KeaAppTabComponent,
@@ -284,7 +286,7 @@ describe('KeaAppTabComponent', () => {
         expect(component.docAnchorFromHookLibrary('kea-dhcp4')).toBeUndefined()
     })
 
-    it('should display no hooks when no app is loaded', () => {
+    it('should display an empty placeholder when no app is loaded', () => {
         // Check legend.
         const hooksFieldset = fixture.debugElement.query(By.css('#hooks-fieldset'))
         expect(hooksFieldset).toBeTruthy()
@@ -295,7 +297,7 @@ describe('KeaAppTabComponent', () => {
         expect(div).toBeTruthy()
         const divElement = div.nativeElement
         expect(divElement).toBeTruthy()
-        expect(divElement.innerText).toEqual('no hooks')
+        expect(divElement.innerText).toEqual('No hooks')
     })
 
     it('should display hook libraries', () => {
